/******************************************************************************
* @file    PT32x033x_pwm.h
* @author  ӦÿŶ
* @version V1.4.0
* @date    2023/10/27
* @brief    This file contains all the functions prototypes for the PWM firmware library.
*
******************************************************************************
* @attention
*
*
*****************************************************************************/


/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef PT32x033x_PWM_H
#define PT32x033x_PWM_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "PT32x033x.h"

/** @addtogroup PWM
  * @{
  */

/* Exported types ------------------------------------------------------------*/


/** @brief Definition of PWM timebase init structure. **/
typedef struct
{
	u16	PWM_Period;         /*!< Period (Value for MR0 register)             */
	u8	PWM_ClockSel;              /*!< Counting Clock select:HCLK or PCLK          */
	u8 	PWM_BKISel;              /*!< Counting BKI Select:GPIO or LOCKUP or HCLK Fail or PLL Fail or COMP0 or COMP1 or LVD       */
	u8	PWM_Prescaler;             /*!< Prescaler (Value for PR register)           */
	u8	PWM_CounterMode;           /*!< Counter mode up-counting or down-counting   */
	u8	PWM_CounterDirRvEN;       /*!< Counter Direction Reverse enable or disable */
} PWM_TimeBaseInitTypeDef;


/** @brief  PWM Init Structure definition **/
#define PCLK		((u32)0x00000000)
#define HCLK		((u32)0x00000004)

#define CAPTURE_FE	((u32)0x00000000)
#define CAPTURE_RE	((u32)0x00000001)

/** @brief Enumeration of PWM channel. **/
typedef enum
{
	PWM_CH_1 = 0,                /*!< PWM channel 1 */
	PWM_CH_2,                    /*!< PWM channel 2 */
	PWM_CH_3,                    /*!< PWM channel 3 */
	PWM_CH_4                     /*!< PWM channel 4 */
} PWM_CH_Enum;

/** @brief Enumeration of PWM channel output mode. **/
typedef enum
{
	NO_OUTPUT   = 0,                    /*!< PWM channel no output */
	COMPAER_MODE1,                      /*!< PWM channel output high when tc <= compare value,output low when tc > compare value */
	COMPAER_MODE2,                      /*!< PWM channel output low when tc <= compare value,output high when tc > compare value */
	COMPAER_MODE3                       /*!< PWM channel output reverse when tc match compare value */
} PWM_OM_Enum;

/** @brief Definition of PWM channel output init structure. **/
typedef struct
{
	PWM_CH_Enum       Channel;              /*!< Channel selection refer to \ref PWM_CH_Enum */
	PWM_OM_Enum       OutputMode;           /*!< Channel output mode selection refer to \ref MCTM_OM_Enum */
	u32          ControlN;             /*!< CHxN output enble or disable refer to */
	u32          IdleState;            /*!< CHx  Idle State refer to */
	u16          CompareValue;         /*!< Value for MRx register */
	uint8_t           DeathTime;            /*!< Value for DT register */
} PWM_OutputInitTypeDef;

typedef enum
{
	Rise_Mode=0,
	Fall_Mode,
	Reset_Enable
}PWM_IN_Enum;

typedef struct
{
	PWM_CH_Enum       Channel;              /*!< Channel selection refer to \ref PWM_CH_Enum */
	PWM_IN_Enum       InMode;           /*!< Channel output mode selection refer to \ref MCTM_OM_Enum */
	PWM_IN_Enum       ResetCNT;
} PWM_InputInitTypeDef;

/** @brief Enumeration of PWM channel Capture mode. **/
#define CHAN_CAPTURE_RiseEdage_SEL	((u16)0x00000001)
#define CHAN_CAPTURE_FallingEdage_SEL	((u16)0x00000002)
#define CHAN_CAPTURE_ResetTC_SEL	((u16)0x00000004)
#define CHAN_CAPTURE_Interrupt_Enable	((u16)0x00000008)
#define CHAN_CAPTURE_Interrupt_Disable	((u16)0x00000000)


/** @brief Definition of PWM channel Capture init structure. **/
typedef struct
{
	PWM_CH_Enum       Channel;              /*!< Channel selection refer to \ref PWM_CH_Enum */
	u16          CaptureMode;           /*!< Channel Capture mode selection*/
	u16          Capture_Interrupt;	 /*!< Channel Capture Interrupt selection*/
} PWM_CaptureInitTypeDef;

/* Exported constants --------------------------------------------------------*/
#define CAPTURE_RISEEDAGE_F	((u32)0x00000001)
#define CAPTURE_FALLEDAGE_F	((u32)0x00000002)

/** @brief  PWM channel_N  control. **/
#define  PWM_CHNCTL_DISABLE           ((u32)0x00000000)      /*!< PWM channel_N disable */
#define  PWM_CHNCTL_ENABLE             PWM_OCMR_OC1NE           /*!< PWM channel_N enable */

/** @brief  PWM channel output idle state. **/
#define  PWM_IDLE_LOW                  ((u32)0x00000000)     /*!< PWM channel idle output low  */
#define  PWM_IDLE_HIGH                 PWM_OCMR_OIS1          /*!< PWM channel idle output high  */

/** @defgroup PWM_Count Mode **/
#define PWM_CNT_MODE_UP                ((u32)0x00000000)
#define PWM_CNT_MODE_DOWN              ((u32)0x00000040)
#define PWM_CounterDirRv_EN            ((u32)0x00000080)
#define PWM_CounterDirRv_DIS           ((u32)0x00000000)

#define PWM_INT_MR0                    PWM_CR2_ARI
#define PWM_INT_MR1                    PWM_CR2_OC1I
#define PWM_INT_MR2                    PWM_CR2_OC2I
#define PWM_INT_MR3                    PWM_CR2_OC3I
#define PWM_INT_MR4                    PWM_CR2_OC4I
#define PWM_INT_CAP1                   PWM_CAPR_IC1I
#define PWM_INT_CAP2                   PWM_CAPR_IC2I
#define PWM_INT_CAP3                   PWM_CAPR_IC3I
#define PWM_INT_CAP4                   PWM_CAPR_IC4I
#define PWM_INT_BK                     PWM_OCMR_BKI

#define PWM_FLAG_MR0                   PWM_SR_ARF
#define PWM_FLAG_MR1                   PWM_SR_OC1F
#define PWM_FLAG_MR2                   PWM_SR_OC2F
#define PWM_FLAG_MR3                   PWM_SR_OC3F
#define PWM_FLAG_MR4                   PWM_SR_OC4F
#define PWM_FLAG_CAP1_RE               PWM_SR_IC1R
#define PWM_FLAG_CAP1_FE               PWM_SR_IC1F
#define PWM_FLAG_CAP2_RE               PWM_SR_IC2R
#define PWM_FLAG_CAP2_FE               PWM_SR_IC2F
#define PWM_FLAG_CAP3_RE               PWM_SR_IC3R
#define PWM_FLAG_CAP3_FE               PWM_SR_IC3F
#define PWM_FLAG_CAP4_RE               PWM_SR_IC4R
#define PWM_FLAG_CAP4_FE               PWM_SR_IC4F
#define PWM_FLAG_BK                    PWM_OCMR_BKI


#define PWM_BKI_HIGHLEVEL              ((u32)0x02000000)
#define PWM_BKI_LOWLEVEL               ((u32)0x00000000)
#define IS_PWM_BIK_LEVEL(LEVEL)        (((LEVEL) == PWM_BKI_HIGHLEVEL) || \
                                        ((LEVEL) == PWM_BKI_LOWLEVEL))

#define IS_PWM_ALL_PERIPH(PERIPH)            (((PERIPH) == TIM0) || \
        ((PERIPH) == TIM1))

#define IS_PWM_REPEATTIME(TIMES)             ((TIMES) <= 0xF)

#define IS_PWM_COUNTER_RELOAD(RELOADVALUE)   ((RELOADVALUE) <= 0xFFFF)

#define IS_PWM_COUNTER_VALUE(COUNTERVALUE)   ((COUNTERVALUE) <= 0xFFFF)

#define IS_PWM_CLOCKSEL(CLOCKSEL)            (((CLOCKSEL) == PCLK) || \
        ((CLOCKSEL) == HCLK))

#define BKISel_GPIO		((u32)0x00000000)
#define BKISel_HCLK_Fail	((u32)0x00000020)
#define BKISel_PLL_Fail	((u32)0x00000030)
#define BKISel_COMP0	((u32)0x00000040)
#define BKISel_COMP1	((u32)0x00000050)
#define BKISel_PVD		((u32)0x00000060)

#define BKI(Source)	(((Source) == BKISel_GPIO) || \
       				((Source) == BKISel_HCLK_Fail)|| \
       				((Source) == BKISel_PLL_Fail)|| \
       				((Source) == BKISel_COMP0)|| \
       				((Source) == BKISel_COMP1)|| \
       				((Source) == BKISel_PVD))


#define IS_PWM_PRESCALER(PRESCALER)           ((PRESCALER) <= 0xFF)

#define IS_PWM_COUNTERMODE(COUNTERMODE)      (((COUNTERMODE) == PWM_CNT_MODE_UP) || \
        ((COUNTERMODE) == PWM_CNT_MODE_DOWN))

#define IS_PWM_COUNTERDIRRV_EN(COUNTERDIRRV_EN) (((COUNTERDIRRV_EN) == PWM_CounterDirRv_EN) || \
        ((COUNTERDIRRV_EN) == PWM_CounterDirRv_DIS))

#define IS_PWM_CHCHANNEL(CHANNEL)            (((CHANNEL) == PWM_CH_1) || ((CHANNEL) == PWM_CH_2) ||((CHANNEL) == PWM_CH_3) || ((CHANNEL) == PWM_CH_4))

#define IS_PWM_OM(MODE)                      (((MODE) == NO_OUTPUT)     || ((MODE) == COMPAER_MODE1) || \
        ((MODE) == COMPAER_MODE2) || ((MODE) == COMPAER_MODE3))
#define IS_PWM_CM(MODE)                      (((MODE) == CHAN_CAPTURE_RiseEdage_SEL)     || ((MODE) == CHAN_CAPTURE_FallingEdage_SEL) || \
        ((MODE) == CHAN_CAPTURE_ResetTC_SEL) || ((MODE) == (CHAN_CAPTURE_FallingEdage_SEL | CHAN_CAPTURE_RiseEdage_SEL)) )
#define IS_PWM_CMINT(INTSEL)                 (((INTSEL) == CHAN_CAPTURE_Interrupt_Enable) || (INTSEL) == CHAN_CAPTURE_Interrupt_Disable)

#define IS_PWM_CHNCTL(CHNCTL)                (((CHNCTL) == PWM_CHNCTL_DISABLE) || \
        ((CHNCTL) == PWM_CHNCTL_ENABLE))

#define IS_PWM_IDLE(IDLE)                    (((IDLE) == PWM_IDLE_LOW)    || \
        ((IDLE) == PWM_IDLE_HIGH))

#define IS_PWM_COMPAREVALUE(VALUE)           ((VALUE) <= 0xFFFF)

#define IS_PWM_DEATHTIME(VALUE)              ((VALUE) <= 0xFF)

#define IS_PWM_PWM_INT(PWM_INT)              (((PWM_INT) == PWM_INT_MR0)  || ((PWM_INT) == PWM_INT_MR1)  || \
                                              ((PWM_INT) == PWM_INT_MR2)  || ((PWM_INT) == PWM_INT_MR3)  || \
                                              ((PWM_INT) == PWM_INT_MR4)  || ((PWM_INT) == PWM_INT_BK))
																							
#define IS_PWM_CAP_INT(PWM_CAP_INT)			 (((PWM_CAP_INT) ==PWM_INT_CAP1) ||	((PWM_CAP_INT) ==PWM_INT_CAP2)	||\
                                              ((PWM_CAP_INT) ==PWM_INT_CAP3) || ((PWM_CAP_INT) ==PWM_INT_CAP4))

#define IS_PWM_PWM_FLAG(PWM_FLAG)            (((PWM_FLAG) == PWM_FLAG_MR0)     || ((PWM_FLAG) == PWM_FLAG_MR1)     || \
        ((PWM_FLAG) == PWM_FLAG_MR2)     || ((PWM_FLAG) == PWM_FLAG_MR3)     || \
        ((PWM_FLAG) == PWM_FLAG_MR4)     || ((PWM_FLAG) == PWM_FLAG_CAP1_RE) || \
        ((PWM_FLAG) == PWM_FLAG_CAP1_FE) || ((PWM_FLAG) == PWM_FLAG_CAP2_RE) || \
        ((PWM_FLAG) == PWM_FLAG_CAP2_FE) || ((PWM_FLAG) == PWM_FLAG_CAP3_RE) || \
        ((PWM_FLAG) == PWM_FLAG_CAP3_FE) || ((PWM_FLAG) == PWM_FLAG_CAP4_RE) || \
        ((PWM_FLAG) == PWM_FLAG_CAP4_FE) || ((PWM_FLAG) == PWM_FLAG_BK))
#define IS_PWM_CLEAR_FLAG(FLAG)     ((FLAG)  <= 0X3FFF)


/* Exported macro ------------------------------------------------------------*/
/* Exported functions ---------------------------------------------------------*/
void PMW_TimeBaseInit(TIM1_TypeDef* PWMx, PWM_TimeBaseInitTypeDef* TimeBaseInit);
void PWM_TimeBaseStructInit(PWM_TimeBaseInitTypeDef* TimeBaseInit);
void PWM_OutputInit(TIM1_TypeDef* PWMx, PWM_OutputInitTypeDef* OutInit);
void PWM_INT_RepeatTimes(TIM1_TypeDef* PWMx, uint8_t RepeatTimes);
void PWM_ClockSourceConfig(TIM1_TypeDef* PWMx, uint8_t PWM_ClockSel);
void PWM_PrescalerConfig(TIM1_TypeDef* PWMx, uint8_t Prescaler);
void PWM_SetCounterReload(TIM1_TypeDef* PWMx, u16 Reload);
void PWM_SetCounter(TIM1_TypeDef* PWMx, u16 Counter);
void PWM_CounterModeConfig(TIM1_TypeDef* PWMx, uint8_t CounterMode, uint8_t DirRv_EN);
void PWM_UpdateCmd(TIM1_TypeDef* PWMx);
void PWM_Cmd(TIM1_TypeDef* PWMx, FunctionalState NewState);
void PWM_IntConfig(TIM1_TypeDef* PWMx, u32 PWM_INT, FunctionalState NewState);
void PWM_ClearFlag(TIM1_TypeDef* PWMx, u32 PWM_FLAG);
void PWM_RE_CaptureConfig(TIM1_TypeDef* PWMx, uint8_t CHx, FunctionalState NewState);
void PWM_FE_CaptureConfig(TIM1_TypeDef* PWMx, uint8_t CHx, FunctionalState NewState);
void PWM_RST_CaptureConfig(TIM1_TypeDef* PWMx, uint8_t CHx, FunctionalState NewState);
void PWM_OD_Config(TIM1_TypeDef* PWMx, uint8_t CHx, u32 Mode);
void PWM_IDLE_Config(TIM1_TypeDef* PWMx, uint8_t CHx, u32 Idle);
void PWM_CHN_Config(TIM1_TypeDef* PWMx, uint8_t CHx, u32 CHNCTL);
void PWM_DT_Config(TIM1_TypeDef* PWMx, uint8_t CHx, uint8_t DeathTime);
void PWM_SetCompare(TIM1_TypeDef* PWMx, uint8_t CHx, u16 Cmp);
FlagStatus PWM_GetFlagStatus(TIM1_TypeDef* PWMx, u32 PWM_FLAG);
u16 PWM_GetCaptureCounter(TIM1_TypeDef* PWMx, uint8_t CHx);
u16 PWM_GetCaptureEdge(TIM1_TypeDef* PWMx, uint8_t CHx);
u16 PWM_GetCounter(TIM1_TypeDef* PWMx);
u16 PWM_GetPrescaler(TIM1_TypeDef* PWMx);
uint8_t PWM_GetINT_RepeatTimes(TIM1_TypeDef* PWMx);
void PWM_BreakInput_Cmd(TIM1_TypeDef* PWMx, FunctionalState NewState);
void PWM_BKI_LevelConfig(TIM1_TypeDef* PWMx, u32 BKI_Level);
void PWM_CapITConfig(TIM1_TypeDef* PWMx,u32 PWM_CAP_INT,FunctionalState NewState);
void PWM_CaptureInit(TIM1_TypeDef* PWMx, PWM_CaptureInitTypeDef* CapInit);
void PWM_DeInit(TIM1_TypeDef* PWMx);
/**
  * @}
  */

  
#ifdef __cplusplus
}
#endif

#endif


