/******************************************************************************
  * @file    PT32x033x_adc.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief   This file provides firmware functions to manage the following
  *          functionalities of the ADC peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_adc.h"

/** @defgroup ADC
  * @brief ADC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define INICFR_CLEAR_MASK		((u32)0xFF8C8440)
#define CALIBRATION_TIMEOUT	((u32)0x00000FFF)
#define ADC_DeInit_MASK			((u32)0x110001F)
#define SAMPLE_DeInit_MASK	((u32)0x3)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Deinitializes the ADCx peripheral registers to their default reset values.
  * @param  ADCx: where x can be 1 to select the ADC peripheral.
  * @retval None
  */
void ADC_DeInit(ADC_TypeDef* ADCx)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	ADCx->CR = ADC_DeInit_MASK;
	ADCx->SAMPLE = SAMPLE_DeInit_MASK;
}

/**
  * @brief  Initializes the ADCx peripheral according to the specified parameters
  *         in the ADC_InitStruct.
  * @note   This function is used to configure the global features of the ADC (
  *         Resolution, Data Alignment, continuous mode activation, External
  *         trigger source and edge, Sequence Scan Direction).
  * @param  ADCx: where x can be 1 to select the ADC peripheral.
  * @param  ADC_InitStruct: pointer to an ADC_InitTypeDef structure that contains
  *         the configuration information for the specified ADC peripheral.
  * @retval None
  */
void ADC_Init(ADC_TypeDef* ADCx, ADC_InitTypeDef* ADC_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_MODE_SEL(ADC_InitStruct->ADC_MODE));
	assert_param(IS_ADC_TIMS_SEL(ADC_InitStruct->ADC_TIMS));
	assert_param(IS_ADC_ALIGN_SEL(ADC_InitStruct->ADC_ALIGN));
	assert_param(IS_ADC_CHS_SEL(ADC_InitStruct->ADC_Channel));
	assert_param(IS_ADC_BGS_SEL(ADC_InitStruct->ADC_AnalogChannel));
	assert_param(IS_ADC_ADVRPS_SEL(ADC_InitStruct->ADC_ReferencePositive));
	assert_param(IS_ADC_PSC_SEL(ADC_InitStruct->ADC_Prescaler));
	
	/* Get the ADCx CFGR value */
	tmpreg = ADCx->CR;
	/* Clear SCANDIR, RES[1:0], ALIGN, EXTSEL[2:0], EXTEN[1:0] and CONT bits */
	tmpreg &= INICFR_CLEAR_MASK;
	/*---------------------------- ADCx CFGR Configuration ---------------------*/
	/* Set MODE bit according to ADC_MODE value */
	/* Set EXTSEL[5:2] bits according to ADC_TIMS value */
	/* Set DISH bit according to ADC_OffsetCalibration value */
	/* Set ALIGN bit according to ADC_ALIGN value */
	tmpreg  |= (ADC_InitStruct->ADC_MODE) |
	           	(ADC_InitStruct->ADC_ALIGN)|
							(ADC_InitStruct->ADC_Channel)|
							(ADC_InitStruct->ADC_TIMS)|
							(ADC_InitStruct->ADC_ReferencePositive)|
							(ADC_InitStruct->ADC_Prescaler)|
							(ADC_InitStruct->ADC_AnalogChannel);
	/* Write to ADCx CFGR */
	ADCx->CR = tmpreg;
	ADCx->SAMPLE=0x1F;
}

/**
  * @brief  Fills each ADC_InitStruct member with its default value.
  * @note   This function is used to initialize the global features of the ADC (
  *         Resolution, Data Alignment, continuous mode activation, External
  *         trigger source and edge, Sequence Scan Direction).
  * @param  ADC_InitStruct: pointer to an ADC_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void ADC_StructInit(ADC_InitTypeDef* ADC_InitStruct)
{
	ADC_InitStruct->ADC_Prescaler = ADC_CLK_DIV2;							//ADCƵ8/2=4 MHz
	ADC_InitStruct->ADC_MODE = ADC_MODE_Single;						//תģʽ
	ADC_InitStruct->ADC_TIMS=ADC_TIMS_TIM0Trgo;						//ʱԴѡTIM0¼
	ADC_InitStruct->ADC_ALIGN = ADC_DataAlign_Right;					//Ҷ
	ADC_InitStruct->ADC_Channel=ADC_Channel_0;						//ͨ0,PC13
	ADC_InitStruct->ADC_ReferencePositive= ADC_ReferencePositive_VDDA;	//ѡVDDAΪ˲οƽ
	ADC_InitStruct->ADC_AnalogChannel=ADC_AnalogChannel_BG1v2;
}

/**
  * @brief  Enables or disables the specified ADC peripheral.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral.
  * @param  NewState: new state of the ADCx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_Cmd(ADC_TypeDef* ADCx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Set the ADEN bit to Enable the ADC peripheral */
		ADCx->CR |= ADC_CR_EN;
	}
	else
	{
		/* Reset the ADEN to Disable the ADC peripheral */
		ADCx->CR &=(~ADC_CR_EN);
	}
}


/**
  * @brief  Set ADC sample times the specified ADC peripheral.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral.
  * @param  sampletime: sample time value (3~255).
  * @retval None
  */
void ADC_SetSampleTime(ADC_TypeDef* ADCx, u32 sampletime)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_SMP_SEL(sampletime));
	ADCx->SAMPLE = sampletime;
}

/**
  * @brief  Configures for the selected ADC and its sampling time.
  * @param  ADCx: where x can be 1 to select the ADC peripheral.
  * @param  ADC_Channel: the ADC channel to configure.
  *          This parameter can be any combination of the following values:
  *            @arg ADC_Channel_0: ADC Channel0 selected
  *            @arg ADC_Channel_1: ADC Channel1 selected
  *            @arg ADC_Channel_2: ADC Channel2 selected
  *            @arg ADC_Channel_3: ADC Channel3 selected
  *            @arg ADC_Channel_4: ADC Channel4 selected
  *            @arg ADC_Channel_5: ADC Channel5 selected
  *            @arg ADC_Channel_6: ADC Channel6 selected
  *            @arg ADC_Channel_7: ADC Channel7 selected
  *            @arg ADC_Channel_8: ADC Channel8 selected
  *            @arg ADC_Channel_9: 
  *            @arg ADC_Channel_10: 
  *            @arg ADC_Channel_11:
  * @retval None
  */
void ADC_ChannelConfig(ADC_TypeDef* ADCx, u32 ADC_Channel)
{
	u32 tmpreg=0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_CHS_SEL(ADC_Channel));
	
	tmpreg = ADCx->CR;
	tmpreg &= (u32)~ADC_CR_CHS;
	tmpreg |= ADC_Channel;
	ADCx->CR = tmpreg;
}

/**
  * @brief  Start Conversion for the selected ADC channels.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral.
  * @retval None
  */
void ADC_ConversionConfig(ADC_TypeDef* ADCx,FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		ADCx->CR |= (u32)ADC_CR_SOC;
	}
	else
	{
		// None
	}
}


/**
  * @brief  Returns the last ADCx conversion result data for ADC channel.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral.
  * @retval The Data conversion value.
  */
u16 ADC_GetConversionValue(ADC_TypeDef* ADCx)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	/* Return the selected ADC conversion value */
	return (u16) ADCx->DR;
}

/**
  * @brief  Checks whether the specified ADC flag is set or not.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral.
  * @param  ADC_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg ADC_FLAG_EOC: End of conversion flag
  *            @arg ADC_FLAG_RDY: ADC Ready flag
  * @retval The new state of ADC_FLAG (SET or RESET).
  */
FlagStatus ADC_GetFlagStatus(ADC_TypeDef* ADCx, u32 ADC_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_GET_FLAG(ADC_FLAG));
	/* Check the status of the specified ADC flag */
	if ((ADCx->SR & ADC_FLAG) != (u32)RESET)
	{
		/* ADC_FLAG is set */
		bitstatus = SET;
	}
	else
	{
		/* ADC_FLAG is reset */
		bitstatus = RESET;
	}
	/* Return the ADC_FLAG status */
	return  bitstatus;
}


/**
  * @}
  */
