/******************************************************************************
  * @file    PT32x033x_dac.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the DAC peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

  
/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_dac.h"


/** @defgroup DAC
  * @brief DAC driver modules
  * @{
  */
#define DAC_FLAG_EOC DAC_SR_EOC
#define DAC_Enable DAC_Prescaler_EN
#define DAC_CR_MASK	((u32)0xFFFFFC8E)

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

  /**
  * @brief  Initializes the DAC peripheral according to the specified parameters
  *         in the DAC_InitStruct.
  * @note   This function is used to configure the global features of the DAC (
  *         DAC CLK, DAC REF Source).
  * @param  DAC_InitStruct: pointer to an DAC_InitStruct structure that contains
  *         the configuration information for the specified DAC peripheral.
  * @retval None
  */
void DAC_Init(DAC_InitTypeDef* DAC_InitStruct)
{
	u32 tmpreg = 0;
	assert_param(IS_DAC_CLK_SEL(DAC_InitStruct->DAC_Prescaler));
	assert_param(IS_DAC_PREF_SEL(DAC_InitStruct->DAC_RefVoltage));
	tmpreg = DAC->CR;
	tmpreg &= DAC_CR_MASK;
	tmpreg  |=	DAC_InitStruct->DAC_Prescaler|DAC_InitStruct->DAC_RefVoltage;
	DAC->CR=tmpreg;
}

/**
  * @brief  Fills each DAC_InitStruct member with its default value.
  * @note   This function is used to initialize the global features of the DAC (
  *         DAC CLK, DAC REF Source).
  * @param  DAC_InitStruct: pointer to an DAC_InitStruct structure which will
  *         be initialized.
  * @retval None
  */
void DAC_StructInit(DAC_InitTypeDef* DAC_InitStruct)
{
	DAC_InitStruct->DAC_Prescaler=DAC_Prescaler_40;
	DAC_InitStruct->DAC_RefVoltage = DAC_RefVoltage_BG2V0;
}

/**
  * @brief  Set the DAC Output Value.
  * @param  Value: DAC Output Value can be 0 or 4095.
  * @retval None.
  */
void DAC_SetData(u16 Value)
{
	DAC->DR=Value;
}

/**
  * @brief  Enables or disables the specified DAC peripheral.
  * @param  NewState: new state of the DAC peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void DAC_Cmd(FunctionalState NewState)
{
	if (NewState != DISABLE)
	{
		DAC->CR |= (u32)DAC_Enable;
	}
	else
	{
		DAC->CR &=((u32)~DAC_Enable);
	}
}

/**
  * @brief  Enables or disables the specified DAC interrupts.
  * @param  NewState: new state of the specified DAC interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void DAC_ITConfig(FunctionalState NewState)
{
	if (NewState != DISABLE)
	{
		DAC->IE |= DAC_IE_EOCE;
	}
	else
	{
		DAC->IE &= (u16)~DAC_IE_EOCE;
	}
}

/**
  * @brief  Clears the DAC pending flags.
  * @retval None
  */
void DAC_ClearFlag(void)
{
	DAC->SR |= DAC_FLAG_EOC;
}

/**
  * @brief  Checks whether the specified DAC flag is set or not.
  * @retval The new state of DAC FLAG (SET or RESET).
  */
FlagStatus DAC_GetFlagStatus(void)
{
	FlagStatus bitstatus = RESET;
	if ((DAC->SR & 0X01) != (u16)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return bitstatus;
}

/**
  * @brief  wait for DAC conversion to OK
  * @retval None
  */
void DAC_Wait_For_Transformat_OK(void)
{
	while ((DAC->SR & DAC_FLAG_EOC) == (u16)RESET)
	{
	}
}

void DAC_DeInit(void)
{
	DAC->CR=(u32)0x0;
	DAC->IE=(u32)0x0;
}

/**
  * @}
  */
