  /******************************************************************************
  * @file    PT32x033x_gpio.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the GPIO peripheral:
  *           + Initialization and Configuration functions
  *           + GPIO Read and Write functions
  *           + GPIO Alternate functions configuration functions
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_gpio.h"


/** @defgroup GPIO
  * @brief GPIO driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Initializes the GPIOx peripheral according to the specified
  *         parameters in the GPIO_InitStruct.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_InitStruct: pointer to a GPIO_InitTypeDef structure that contains
  *         the configuration information for the specified GPIO peripheral.
  * @retval None
  */
void GPIO_Init(GPIO_TypeDef* GPIOx, GPIO_InitTypeDef* GPIO_InitStruct)
{
	
	u32 currentpin = 0x00;
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_SEL(GPIO_InitStruct->GPIO_Pin));
	assert_param(IS_GPIO_PUPD(GPIO_InitStruct->GPIO_PuPd));
	/*-------------------------- Configure the port pins -----------------------*/
	/*-- GPIO Mode Configuration --*/
	/* Get the port pins position */
	currentpin = GPIO_InitStruct->GPIO_Pin;
	if (((GPIO_InitStruct->GPIO_Mode) == GPIO_Mode_OutPP) || ((GPIO_InitStruct->GPIO_Mode) == GPIO_Mode_OutOD))
	{
		/* Output mode configuration */
		GPIOx->OES = currentpin;
		if((GPIO_InitStruct->GPIO_Mode) == GPIO_Mode_OutOD)
		{
			GPIOx->ODS = currentpin;
		}
		else
		{
			GPIOx->ODC = currentpin;
		}
	}
	if((GPIO_InitStruct->GPIO_Mode) == GPIO_Mode_IN)
	{
		/* input mode configuration */
		GPIOx->OEC = currentpin;
		switch(((u32)GPIO_InitStruct->GPIO_PuPd))
		{
			case GPIO_PuPd_NOPULL :
				GPIOx->PDC = currentpin;
				GPIOx->PUC = currentpin;
				break;
			case GPIO_PuPd_UP :
				GPIOx->PDC = currentpin;
				GPIOx->PUS = currentpin;
				break;
			case GPIO_PuPd_DOWN :
				GPIOx->PDS = currentpin;
				GPIOx->PUC = currentpin;
				break;
		}
	}
}


/**
  * @brief  Fills each GPIO_InitStruct member with its default value.
  * @param  GPIO_InitStruct: pointer to a GPIO_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void GPIO_StructInit(GPIO_InitTypeDef* GPIO_InitStruct)
{
	/* Reset GPIO init structure parameters values */
	GPIO_InitStruct->GPIO_Pin  = GPIO_Pin_0;
	GPIO_InitStruct->GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStruct->GPIO_PuPd = GPIO_PuPd_NOPULL;
}



/**
  * @brief  Reads the specified input port pin.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bit to read.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~15)
  * @retval The input port pin value.
  */
u8 GPIO_ReadInputDataBit(GPIO_TypeDef* GPIOx, u16 GPIO_Pin)
{
	u8 bitstatus = 0x00;
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_GET(GPIO_Pin));
	if ((GPIOx->DR & GPIO_Pin) != (u32)Bit_RESET)
	{
		bitstatus = (u8)Bit_SET;
	}
	else
	{
		bitstatus = (u8)Bit_RESET;
	}
	return bitstatus;
}

/**
  * @brief  Reads the specified input port 
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @retval The input port value.
  */
u16 GPIO_ReadInputData(GPIO_TypeDef* GPIOx)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	return ((u16)GPIOx->DR);
}

/**
  * @brief  Reads the specified output data port bit.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: Specifies the port bit to read.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~15)
  *
  *
  * @retval The output port pin value.
  */
u8 GPIO_ReadOutputDataBit(GPIO_TypeDef* GPIOx, u16 GPIO_Pin)
{
	u8 bitstatus = 0x00;
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_GET(GPIO_Pin));
	if ((GPIOx->DR & GPIO_Pin) != (u32)Bit_RESET)
	{
		bitstatus = (u8)Bit_SET;
	}
	else
	{
		bitstatus = (u8)Bit_RESET;
	}
	return bitstatus;
}

/**
  * @brief  Reads the specified GPIO output data port.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @retval GPIO output data port value.
  */
u16 GPIO_ReadOutputData(GPIO_TypeDef* GPIOx)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	return ((u16)GPIOx->DR);
}

/**
  * @brief  Sets the selected data port bits.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~15) or GPIO_Pin_All 
  * @retval None
  */
void GPIO_SetBits(GPIO_TypeDef* GPIOx, u32 GPIO_Pin)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_SEL(GPIO_Pin));
	GPIOx->DR |= GPIO_Pin;
}

/**
  * @brief  Reset the selected data port bits.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~15) or GPIO_Pin_All 
  * @retval None
  */
void GPIO_ResetBits(GPIO_TypeDef* GPIOx, u32 GPIO_Pin)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_SEL(GPIO_Pin));
	GPIOx->DR &= ~GPIO_Pin;
}
/**
  * @brief  Reverse the selected data port bits.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~15) or GPIO_Pin_All 
  * @retval None
  */
void GPIO_ReverseBits(GPIO_TypeDef* GPIOx, u32 GPIO_Pin)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_SEL(GPIO_Pin));
	GPIOx->DR ^= GPIO_Pin;
}

/**
  * @brief  Sets or clears the selected data port bit.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bit to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~15) or GPIO_Pin_All 
  * @param  BitVal: specifies the value to be written to the selected bit.
  *               This parameter can be one of the BitAction enumeration values:
  *               @arg Bit_RESET: to clear the port pin
  *               @arg Bit_SET: to set the port pin
  * @retval None
  */
void GPIO_WriteBit(GPIO_TypeDef* GPIOx, u32 GPIO_Pin, BitAction BitVal)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	assert_param(IS_GPIO_PIN_SEL(GPIO_Pin));
	assert_param(IS_GPIO_BIT_ACTION(BitVal));
	if (BitVal != Bit_RESET)
	{
		GPIOx->DR |= GPIO_Pin;
	}
	else
	{
		GPIOx->DR &= ~GPIO_Pin;
	}
}

/**
  * @brief  Writes data to the specified GPIO data port.
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  PortVal: specifies the value to be written to the port output data register.
  * @retval None
  */
void GPIO_Write(GPIO_TypeDef* GPIOx, u16 PortVal)
{
	/* Check the parameters */
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	GPIOx->DR = PortVal;
}


/**
  * @brief  Enable or Disable Specified Analog Pin Remapping
  * @param  AFIOx: where x can be (A, B, C, D) to select the AFIO peripheral.
  * @param  AFIO_Pin: specifies the port bit to be written.
  *               @arg   This parameter can be AFIO_Pin_x where x can be(0~15) or GPIO_Pin_All 
  * @param  NewState: new state of the port pin remapping.
  *               @arg   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void GPIO_AnaRemapConfig(AFIO_TypeDef* AFIOx, u32 AFIO_Pin, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_AFIO_ALL_PERIPH(AFIOx));
	assert_param(IS_AFIO_PIN_SEL(AFIO_Pin));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		AFIOx->ANAS |= AFIO_Pin;
	}
	else
	{
		AFIOx->ANAC |= AFIO_Pin;
	}
}

/**
  * @brief  Enable or Disable Specified Digital Pin Remapping
  * @param  AFIOx: where x can be (A, B, C, D) to select the AFIO peripheral.
  * @param  AFIO_Pin: specifies the pin for the Alternate function.
  *               @arg   This parameter can be AFIO_Pin_x where x can be(0~15) or GPIO_Pin_All 
  * @param  GPIO_AF: selects the pin to used as Digtal Alternate function.
  *               This parameter can be one of the following value:
  *               @arg This parameter can be AFIO_Pin_x where x can be(0~6)
  * @param  NewState: new state of the port pin remapping.
  *               @arg   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void GPIO_DigRemapConfig(AFIO_TypeDef* AFIOx, u16 AFIO_Pin, u8 AFIO_AF, FunctionalState NewState)
{
	u8 temp = 0;
	/* Check the parameters */
	assert_param(IS_AFIO_ALL_PERIPH(AFIOx));
	assert_param(IS_AFIO_PIN_SEL(AFIO_Pin));
	assert_param(IS_AFIO_AF_SEL(AFIO_AF));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		AFIOx->AFC |= AFIO_Pin;
		while(AFIO_Pin)			
		{
			if((AFIO_Pin&0x01)==1)	
			{
				if(temp<8)
				{
					AFIOx->AFS0|=AFIO_AF<<(temp*4);
				}
				else
				{
					AFIOx->AFS1|=AFIO_AF<<((temp-8)*4);
				}
			}
			AFIO_Pin=AFIO_Pin>>1;
			temp++;				
		}
	}
	else
	{
		AFIOx->AFC |= AFIO_Pin;
	}
}


/**
  * @brief	  Set the selected data port low Byte(PIN0~PIN7),the operation is interrupt safe 
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~7)
  * @retval None
  */
void GPIO_SetLowByte(GPIO_TypeDef* GPIOx, u16 GPIO_Pin)
{
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	
	GPIOx->MASKL[GPIO_Pin & 0xFF] = GPIO_Pin;
}


/**
  * @brief	  Reset the selected data port low Byte(PIN0~PIN7),the operation is interrupt safe 
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~7)
  * @retval None
  */
void GPIO_ResetLowByte(GPIO_TypeDef* GPIOx, u16 GPIO_Pin)
{
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	GPIOx->MASKL[GPIO_Pin & 0xFF] = ~GPIO_Pin;
}


/**
  * @brief	  Set the selected data port High Byte(PIN0~PIN7),the operation is interrupt safe 
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~7)
  * @retval None
  */
void GPIO_SetHighByte(GPIO_TypeDef* GPIOx, u16 GPIO_Pin)
{
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	GPIOx->MASKH[GPIO_Pin & 0xFF] = GPIO_Pin;
}


/**
  * @brief	  Reset the selected data port High Byte(PIN0~PIN7),the operation is interrupt safe 
  * @param  GPIOx: where x can be (A, B, C, D) to select the GPIO peripheral.
  * @param  GPIO_Pin: specifies the port bits to be written.
  *               @arg   This parameter can be GPIO_Pin_x where x can be(0~7)
  * @retval None
  */
void GPIO_ResetHighByte(GPIO_TypeDef* GPIOx, u16 GPIO_Pin)
{
	assert_param(IS_GPIO_ALL_PERIPH(GPIOx));
	GPIOx->MASKH[GPIO_Pin & 0xFF] = ~GPIO_Pin;
}


