/******************************************************************************
  * @file    PT32x033x_ifmc.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the IFMC peripheral:
  *           + Initialization and Configuration
  *           + Flash Erase and Read\Write Function
  *           + Interrupts and flags management
  *          
  ******************************************************************************
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣ ӻ̷𺦸
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_ifmc.h"


/** @defgroup IFMC
  * @brief IFMC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

#define IFMC_CR_MASK 0x0000107

/**
  * @brief  Sets the code latency value.
  * @param  IFMC_Latency: specifies the IFMC Latency value.
  *          This parameter can be one of the following values:
  *             @arg IFMC_LatencyCycle_0: IFMC Zero Latency cycle
  *             @arg IFMC_LatencyCycle_1: IFMC One Latency cycle
  * @retval None
  */
void IFMC_SetLatency(u32 IFMC_Latency)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_FLASH_WAIT_SEL(IFMC_Latency));
	/* Read the ACR register */
	tmpreg = IFMC->CR;
	/* Sets the Latency value */
	tmpreg &= (u32) (~((u32)IFMC_CR_WAIT));
	tmpreg |= IFMC_Latency;
	/* Write the ACR register */
	IFMC->CR = tmpreg;
}

/**
  * @brief  Sets the IFMC Prescaler.
  * @param  Prescaler: specifies the IFMC PSC value.
  *		    This parameter must be less than 64 
  * @retval None
  */
void IFMC_SetPrescaler(u8 Prescaler)
{
	assert_param(IS_IFMC_PSC_Value(Prescaler));
	/* Write the PSC register */
	IFMC->PSC =Prescaler;
}


/**
  * @brief  Returns the IFMC Status.
  * @param  None
  * @retval FLASH Status: The returned value can be:
                 @arg   IFMC_Busy
                 @arg   IFMC_KeyError
                 @arg   IFMC_AddrError
                 @arg   IFMC_Comolete
  */
IFMC_Status IFMC_GetFlashStatus(void)
{
	IFMC_Status IFMCstatus = IFMC_Complete;
	if((IFMC->SR & IFMC_FLAG_BUSY) == IFMC_FLAG_BUSY)
	{
		IFMCstatus = IFMC_Busy;
	}
	else
	{
		if((IFMC->SR & (u32)(IFMC_FLAG_KERR)) != (u32)0x00)
		{
			IFMCstatus = IFMC_KeyError;
		}
		else
		{
			if((IFMC->SR & (u32)(IFMC_FLAG_AERR)) != (u32)0x00)
			{
				IFMCstatus = IFMC_AddrError;
			}
			else
			{
				IFMCstatus = IFMC_Complete;
			}
		}
	}
	/* Return the IFMC Status */
	return IFMCstatus;
}


/**
  * @brief  Waits for a IFMC operation to complete or a TIMEOUT to occur.
  * @param  Timeout: IFMC programming Timeout
  * @retval IFMC Status: The returned value can be: IFMC_Busy,
  *         IFMC_ERROR_PROGRAM, IFMC_ERROR_WRP, IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_WaitForLastOperation(u32 Timeout)
{
	IFMC_Status status = IFMC_Complete;
	/* Check for the IFMC Status */
	status = IFMC_GetFlashStatus();
	/* Wait for a IFMC operation to complete or a TIMEOUT to occur */
	while((IFMC_GetFlashStatus() == IFMC_Busy) && (Timeout != 0x00))
	{
		status = IFMC_GetFlashStatus();
		Timeout--;
	}
	if(Timeout == 0x00 )
	{
		status = IFMC_TimeOut;
	}
	/* Return the operation status */
	return status;
}


/**
  * @brief  Erases a specified page in program memory.
  * @note   To correctly run this function, the IFMC_Unlock() function must be called before.
  * @note   Call the IFMC_Lock() to disable the flash memory access (recommended
  *         to protect the IFMC memory against possible unwanted operation)
  * @param  Page_Address: The page address in program memory to be erased.
  * @note   A Page is erased in the Program memory only if the address to load
  *         is the start address of a page (multiple of 1024 bytes).
  * @retval IFMC Status: The returned value can be:
  *         IFMC_ERROR_PROGRAM, IFMC_ERROR_WRP, IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_ErasePage(u32 Page_Address)
{
	u32 tmpreg = 0;
	IFMC_Status status = IFMC_Complete;
	/* Check the parameters */
	assert_param(IS_IFMC_PROGRAM_ADDRESS(Page_Address));
	/* Wait for last operation to be completed */
	status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);	//ȴһ
	tmpreg = IFMC->CR;
	tmpreg &= ~ (IFMC_CR_MASK);
	if(status == IFMC_Complete)
	{
		IFMC->AR  = Page_Address;
		if(((Page_Address) < MAINCODE_END_ADDR))	//ַСBoot ŵַΧ(ڵĵַ)
		{
			/* The main code area is erased by page */
			tmpreg |= IFMC_MainCodeKey | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
			IFMC->CR =tmpreg;
			status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
		}
		else											//ַBoot ŵַΧ(ûϢڵĵַ)
		{
			/* User information area is erased by page */
			if( ((Page_Address) > EncryptedArea_StartAddr) && ((Page_Address) < EncryptedArea_EndAddr))
			{
				if( ((Page_Address) > EncryptedArea0_StartAddr) && ((Page_Address) < EncryptedArea0_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea0Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea1_StartAddr) && ((Page_Address) < EncryptedArea1_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea1Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea2_StartAddr) && ((Page_Address) < EncryptedArea2_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea2Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea3_StartAddr) && ((Page_Address) < EncryptedArea3_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea3Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea4_StartAddr) && ((Page_Address) < EncryptedArea4_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea4Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea5_StartAddr) && ((Page_Address) < EncryptedArea5_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea5Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea6_StartAddr) && ((Page_Address) < EncryptedArea6_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea6Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Page_Address) > EncryptedArea7_StartAddr) && ((Page_Address) < EncryptedArea7_EndAddr))
				{
					tmpreg |= IFMC_EncryptedArea7Key | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
					IFMC->CR=tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else 
				{
					status = IFMC_AddrError;
				}
			}
			else										//bootŵַΧܲ
			{
				status = IFMC_AddrError;
			}
		}
	}
	/* Return the Erase Status */
	return status;
}

/**
  * @brief  Erases all IFMC pages.
  * @note   To correctly run this function, the IFMC_Unlock() function must be called before.
  * @note   Call the IFMC_Lock() to disable the flash memory access (recommended
  *         to protect the IFMC memory against possible unwanted operation)
  * @param  None
  * @retval IFMC Status: The returned value can be: IFMC_ERROR_PG,
  *         IFMC_ERROR_WRP, IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_EraseAllPages(void)
{
	u32 tmpreg = 0;	
	IFMC_Status status = IFMC_Complete;
	tmpreg = IFMC->CR;	
	tmpreg &= ~ (IFMC_CR_MASK);		
	/* Wait for last operation to be completed */
	status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
	if(status == IFMC_Complete)
	{
		/* if the previous operation is completed, proceed to erase all pages */
	  tmpreg |= IFMC_MainCodeKey | IFMC_WriteOperation | IFMC_MainCodeErase | IFMC_OperationStarts;
		IFMC->CR =tmpreg;		
		/* Wait for last operation to be completed */
		status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
	}
	/* Return the Erase Status */
	return status;
}


/**
  * @brief  Programs a word at a specified address.
  * @note   To correctly run this function, the IFMC_Unlock() function must be called before.
  * @note   Call the IFMC_Lock() to disable the flash memory access (recommended
  *         to protect the IFMC memory against possible unwanted operation)
  * @param  Address: specifies the address to be programmed.
  * @param  Data: specifies the data to be programmed.
  * @retval IFMC Status: The returned value can be: IFMC_ERROR_PG,
  *         IFMC_ERROR_WRP, IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_ProgramWord(u32 Address, u32 Data)
{
	IFMC_Status status = IFMC_Complete;
	__IO u32 tmp = 0;
	u32 tmpreg = 0;
	tmpreg = IFMC->CR;	
	tmpreg &= ~ (IFMC_CR_MASK);	
	/* Check the parameters */
	assert_param(IS_IFMC_PROGRAM_ADDRESS(Address));
	/* Wait for last operation to be completed */
	status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
	if(status == IFMC_Complete)
	{
		/* if the previous operation is completed, proceed to erase all pages */
		IFMC->AR  = Address;
		IFMC->DR  = Data;
		if(((Address) < MAINCODE_END_ADDR) && ((Address & 0x03) == 0))
		{
			/* Data is programming into the main code area */
			tmpreg |=IFMC_MainCodeKey | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
			IFMC->CR = tmpreg;
			status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
		}
		else
		{
			/* Data is programming into the user information area */
			if( ((Address) > EncryptedArea_StartAddr) && ((Address) < EncryptedArea_EndAddr) && ((Address & 0x03) == 0))
			{
				if( ((Address) > EncryptedArea0_StartAddr) && ((Address) < EncryptedArea0_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea0Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;					
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea1_StartAddr) && ((Address) < EncryptedArea1_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea1Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea2_StartAddr) && ((Address) < EncryptedArea2_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea2Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea3_StartAddr) && ((Address) < EncryptedArea3_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea3Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea4_StartAddr) && ((Address) < EncryptedArea4_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea4Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea5_StartAddr) && ((Address) < EncryptedArea5_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea5Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea6_StartAddr) && ((Address) < EncryptedArea6_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea6Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else if( ((Address) > EncryptedArea7_StartAddr) && ((Address) < EncryptedArea7_EndAddr))
				{
					tmpreg |=IFMC_EncryptedArea7Key | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
				}
				else 
				{
					status = IFMC_AddrError;
				}
			}
			else
			{
				status = IFMC_AddrError;
			}
		}
	}
	/* Return the Program Status */
	return status;
}


/**
  * @brief  Read a word at a specified address.
  * @note
  * @note
  *
  * @param
  * @note
  *
  * @retval
  *
  */
u32 IFMC_ReadWord(u32 Address)
{
	return *(__IO u32*) Address;
}
/**
  * @brief  Read a half word at a specified address.
  * @note
  * @note
  *
  * @param
  * @param
  * @retval
  *
  */
u16 IFMC_ReadHalfWord(u32 Address)
{
	return *(__IO u16*) Address;
}

/**
  * @brief Read a Byte at a specified address.
  * @note
  * @note
  *
  * @param
  * @param
  * @retval
  *
  */
u8 IFMC_ReadByte(u32 Address)
{
	return *(__IO u8*) Address;
}


/**
  * @brief  Enables or disables the specified IFMC interrupts.
  * @param  IFMC_IT: specifies the IFMC interrupt sources to be enabled or
  *         disabled.
  *          This parameter can be any combination of the following values:
  *             @arg IFMC_IE_WOVI
  *             @arg IFMC_IE_POVI	  
  *             @arg IFMC_IE_COVI	 
  *             @arg IFMC_IE_CERI	
  *             @arg IFMC_IE_KERI	
  *             @arg IFMC_IE_AERI	
  * @retval None
  */
void IFMC_ITConfig(u32 IFMC_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_IFMC_IT(IFMC_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if(NewState != DISABLE)
	{
		/* Enable the interrupt sources */
		IFMC->IE |= IFMC_IT;
	}
	else
	{
		/* Disable the interrupt sources */
		IFMC->IE &= ~(u32)IFMC_IT;
	}
}

/**
  * @brief  Checks whether the specified IFMC flag is set or not.
  * @param  IFMC_FLAG: specifies the IFMC flag to check.
  *          This parameter can be one of the following values:
  *             @arg IFMC_FLAG_BSY: IFMC write/erase operations in progress flag
  *             @arg IFMC_FLAG_PGERR: IFMC Programming error flag flag
  *             @arg IFMC_FLAG_WRPERR: IFMC Write protected error flag
  *             @arg IFMC_FLAG_EOP: IFMC End of Programming flag
  * @retval The new state of IFMC_FLAG (SET or RESET).
  */
FlagStatus IFMC_GetFlagStatus(u32 IFMC_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_IFMC_SR_GET(IFMC_FLAG));
	if((IFMC->SR & IFMC_FLAG) != (u32)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	/* Return the new state of IFMC_FLAG (SET or RESET) */
	return bitstatus;
}



/**
  * @brief  Clears the IFMC's pending flags.
  * @param  IFMC_FLAG: specifies the IFMC flags to clear.
  *          This parameter can be any combination of the following values:
  *             @arg IFMC_FLAG_WOV	IFMC дɱ־
  *             @arg IFMC_FLAG_POV	IFMCɱ־
  *             @arg IFMC_FLAG_COV	IFMCⱣʱɱ־
  *             @arg IFMC_FLAG_BUSY    IFMC־
  *             @arg IFMC_FLAG_CERR   IFMC־
  *             @arg IFMC_FLAG_KERR   IFMC־
  *             @arg IFMC_FLAG_AERR    IFMCַ־
  * @retval None
  */
void IFMC_ClearFlag(u32 IFMC_FLAG)
{
	/* Check the parameters */
	assert_param(IS_IFMC_SR_SEL(IFMC_FLAG));
	/* Clear the flags */
	IFMC->SR = IFMC_FLAG;
}


/**
  * @brief  Returns the Boot Status.
  * @param  None
  * @retval Boot Status. 
  */
RemapStatus IFMC_GetBootStatus(void)
{
	u32 tmp;
	tmp = (IFMC->BSR & IFMC_BSR_BSR);
	if (tmp != RESET)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/**
  * @brief  Returns the ReadProtect Status.
  * @param  None
  * @retval ReadProtect Status
  */
ProtectStatus IFMC_GetReadProtectStatus(void)
{
	u32 tmp;
	tmp = (IFMC->RPT & IFMC_RPT_PSR);
	if (tmp != RESET)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}
/**
  * @brief  Reset Remap Bit.
  * @param  None
  * @retval None
  */
void IFMC_ResetRemapBit(void)
{
	IFMC->BSR = RESET;
}


/**
  * @}
  */


