  /******************************************************************************
  * @file    PT32x033x_lcd.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief        This file provides firmware functions to manage the following
  *          functionalities of the LCD peripheral:
  *           + Initialization and Configuration functions
  *           + LCD Gray Value Set
  *           + LCD COM Configuration functions
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

  
/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_lcd.h"

/** @defgroup LCD
  * @brief LCD driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------------------------------------*/
/* Private define ------------------------------------------------------------------------------------------*/
#define LCD_DeInit_MASK			((u32)0x00008000)
/* Private macro -------------------------------------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------------------------------------*/
u8 LCD_DutyFlag=0;
u32 LCD_duty=0;

/**
  * @brief  
  * @param
  * @retval
  */
void LCD_DeInit(void)
{
	LCD->CR = LCD_DeInit_MASK;
	Wait_LCD();
}


/**
  * @brief  Initializes the LCD peripheral according to the specified parameters in the LCD_InitStruct.
  * @note   This function is used to configure the global features of the LCD 
  * @param  LCD_InitStruct: pointer to an LCD_InitTypeDef structure that contains
  *         the configuration information for the specified LCD peripheral.
  * @retval None
  */
void LCD_Init(LCD_InitTypeDef* LCD_InitStruct)
{
	uint32_t tmpreg = 0;
	
	assert_param(IS_LCD_CRTYPE_SEL(LCD_InitStruct->LCD_DriveVoltageType));
	assert_param(IS_LCD_DUTY_SEL(LCD_InitStruct->LCD_Duty));
	assert_param(IS_LCD_WAVE_SEL(LCD_InitStruct->LCD_WaveControl));
	assert_param(IS_LCD_BLINK_SEL(LCD_InitStruct->LCD_BlinkMode));
	assert_param(IS_LCD_CLK_SEL(LCD_InitStruct->LCD_ClkSource));
	assert_param(IS_LCD_DMOD_SEL(LCD_InitStruct->LCD_PageDisplayMode));
	assert_param(IS_LCD_CAPMODE_SEL(LCD_InitStruct->LCD_ChargePump));
	assert_param(IS_LCD_RES_SEL(LCD_InitStruct->LCD_TotalResistance));
	tmpreg = LCD->CR;
	tmpreg &= 0xFFFF0181;
	tmpreg  |=	LCD_InitStruct->LCD_DriveVoltageType|
				LCD_InitStruct->LCD_Duty|
				LCD_InitStruct->LCD_WaveControl|
				LCD_InitStruct->LCD_BlinkMode|
				LCD_InitStruct->LCD_ClkSource|
				LCD_InitStruct->LCD_PageDisplayMode|
				LCD_InitStruct->LCD_ChargePump|
				LCD_InitStruct->LCD_TotalResistance;
	
	if(LCD_InitStruct->LCD_ClkSource==LCD_ClkSource_LCDI)	tmpreg|=LCD_CR_LCDIE;
	
	LCD->CR=tmpreg;
	Wait_LCD();
}


/**
  * @brief  Fills each LCD_InitStruct member with its default value.
  * @note   This function is used to initialize the global features of the LCD
  * @param  LCD_InitStruct: pointer to an LCD_InitTypeDef structure which will be initialized.
  * @retval None
  */
void LCD_StructInit(LCD_InitTypeDef* LCD_InitStruct)
{
	LCD_InitStruct->LCD_DriveVoltageType=LCD_DriveVoltageType_R;		//Rƫѹ                      
	LCD_InitStruct->LCD_Duty = LCD_Duty_Div4;		// 1/4DUTY,4COM,28SEG
	LCD_InitStruct->LCD_WaveControl = LCD_WaveControl_A;			//A type
	LCD_InitStruct->LCD_BlinkMode= LCD_BlinkMode_Disable;	//ʾģʽ
	LCD_InitStruct->LCD_ClkSource=LCD_ClkSource_LCDI;		//LCDģڲ32khzʱ
	LCD_InitStruct->LCD_PageDisplayMode=LCD_PageDisplayMode_Fixed;	//̶ҳʾ
	LCD_InitStruct->LCD_ChargePump=LCD_ChargePump_Disable;			// VLCD=VDD
	LCD_InitStruct->LCD_TotalResistance=LCD_TotalResistance_45k;		
}

/**
  * @brief		
  *          		
  * @retval None
  */
void LCD_ClearDRAM(void)
{
	u8 i=0,j=0;
	for(i=0;i<16;i++)
	{
		for(j=0;j<8;j++)	LCD_DATA->LCD_PxLy[i*8+j]=0;
	}
}

/**
  * @brief		Clears the LCD Page Buff.
  *          		specifies the Page to clear.
  * @param  Page_Data
  *          		This parameter can be the following values:
  *            @arg LCD_Page_0: Page 0 selected
  *            @arg LCD_Page_1: 
  *            @arg LCD_Page_2:
  *            @arg LCD_Page_3: 
  *            @arg LCD_Page_4: 
  *            @arg LCD_Page_5:
  *            @arg LCD_Page_6:
  *            @arg LCD_Page_7: 
  *            @arg LCD_Page_8:
  *            @arg LCD_Page_9:
  *            @arg LCD_Page_10: 
  *            @arg LCD_Page_11:
  *            @arg LCD_Page_12:
  *            @arg LCD_Page_13: 
  *            @arg LCD_Page_14:
  *            @arg LCD_Page_15:
  * @retval None
  */
void LCD_ClearPage(u32 Page_Data)
{
	u8 j=0;
	assert_param(IS_LCD_Page_SEL(Page_Data));
	for(j=0;j<8;j++)	
	{
		LCD_DATA->LCD_PxLy[Page_Data*8+j]=0;
	}
}

void LCD_SetDramCOM(u32 LCD_PIN,u32 LCD_COM,FunctionalState NEWState)
{
	u8 i,PinOffset,PageOffset,ComOffset;
	assert_param(IS_LCD_PIN_SEL(LCD_PIN));
	assert_param(IS_LCD_COM_SEL(LCD_COM));

	if(NEWState==ENABLE)
	{
		for(i=0;i<16;i++)
		{
			PinOffset=LCD_PIN/4;
			PageOffset=i*8;
			ComOffset=((LCD_PIN%4)*8);
			LCD_DATA->LCD_PxLy[PinOffset+PageOffset]|=LCD_COM<<ComOffset;
		}
	}
	else
	{
		for(i=0;i<16;i++)
		{
			LCD_DATA->LCD_PxLy[LCD_PIN/4+i*8]&=~(LCD_COM<<((LCD_PIN%4)*8));
		}
  }
}

/**
  * @brief  Enables or disables the LCD peripheral.
  * @param  NewState: new state of the LCD peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_CMD(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState == ENABLE)
	{
		if(LCD_DutyFlag)
		{
			LCD->CR &= ~LCD_CR_DUTY;
			LCD->CR |= LCD_duty;
			LCD_DutyFlag = 0;
		}
		LCD->CR |= LCD_CR_EN;//LCD
	}
	else
	{
		LCD_duty=LCD->CR&LCD_CR_DUTY;
		LCD->CR&=~LCD_CR_DUTY;

		LCD->CR &= (~LCD_CR_EN);//رLCD
		LCD_DutyFlag=1;
	}
	Wait_LCD();
}

/**
  * @brief  Enables or disables the LCD Data Update
  * @param  NewState: new state of the LCD Data Update
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_UpdataCMD(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	UNUSED(NewState);
		/* Set the update bit */
	LCD->CR |= LCD_CR_DU;
	while(LCD->CR&LCD_CR_DU);
	Wait_LCD();
}

/**
  * @brief  
  * @param  
  * @retval 
  */
void LCD_AllLightsOnCMD(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState == ENABLE)
	{
		/* Set the update bit */
		LCD->CR |= LCD_CR_AON;
	}
	else
	{
		/* Reset the update bit */
		LCD->CR &= (~LCD_CR_AON);
	}
	Wait_LCD();
}

/**
  * @brief  
  * @param  
  * @retval 
  */
void LCD_AllLightsOffCMD(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState == ENABLE)
	{
		/* Set the update bit */
		LCD->CR |= LCD_CR_AOFF;
	}
	else
	{
		/* Reset the update bit */
		LCD->CR &= (~LCD_CR_AOFF);
	}
	Wait_LCD();
}

/**
  * @brief  Enables or disables the LCD Gray Adj.
  * @param  NewState: new state of the LCD Gray Adj.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_GrayscaleCMD(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState == ENABLE)
	{
		/* Set the update bit */
		LCD->GCR |= LCD_GCR_EN;
	}
	else
	{
		/* Reset the update bit */
		LCD->GCR &= (~LCD_GCR_EN);
	}
	Wait_LCD();
}

/**
  * @brief  Configures for the LCD Grey Value
  * @param  Gray_Rate: the LCD Grey Value to configure.
  *          This parameter can be any combination of the following values:
  *            @arg LCD_Grayscale100: 100% selected
  *            @arg LCD_Grayscale92: 92.4% selected
  *            @arg LCD_Grayscale85: 85.8% selected
  *            @arg LCD_Grayscale80: 80.1% selected
  *            @arg LCD_Grayscale75: 75.2% selected
  *            @arg LCD_Grayscale70: 70.8% selected
  *            @arg LCD_Grayscale66: 66.8% selected
  *            @arg LCD_Grayscale63: 63.3% selected
  * @retval None
  */
void LCD_Grayscale(uint16_t Gray_Rate)
{
	u32 tmpreg = 0;
	assert_param(IS_LCD_GDR(Gray_Rate));
	tmpreg=LCD->GCR;
	tmpreg &= 0xFFFFFFF1;
	tmpreg |=Gray_Rate;
	LCD->GCR=tmpreg;
	Wait_LCD();
}

/**
  * @brief  Configures for the LCD drive voltage type
  * @param  LCD_Type: the LCD Drive Voltage to configure.
  *          This parameter can be any combination of the following values:
  *            @arg LCD_DriveType_C: C Type Drive Voltage
  *            @arg LCD_DriveType_R: R Type Drive Voltage
  * @retval None
  */
void LCD_DriveTypeConfig(u32 LCD_Type)
{
	u32 tmpreg = 0;
	assert_param(IS_LCD_DRIVE_TYPE(LCD_Type));
	tmpreg=LCD->CR;
	tmpreg&=~LCD_CR_TYPE;
	tmpreg|=LCD_Type;
	LCD->CR=tmpreg;
	Wait_LCD();
}

void Wait_LCD()
{
	u32 i=0;
	for(;i<300;i++)
	{;}
}

/**
  * @}
  */
