/******************************************************************************
  * @file    PT32x033x_opa.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief   This file provides firmware functions to manage the following
  *          functionalities of the OPA peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_opa.h"

/** @defgroup OPA
  * @brief OPA driver modules
  * @{
  */
  
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define OPA_CR1_MASK ((u32)0x000000FC)
#define OPA_DeInit_MASK	((u32)0x00000000)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/**
  * @brief  
  * @param  
  * @retval None
  */
void OPA_DeInit(OPA_TypeDef* OPAx)
{
	/* Check the parameters */
	assert_param(IS_OPA_ALL_PERIPH(OPAx));
	OPAx->CR1 = OPA_DeInit_MASK;
	OPAx->CR2 = OPA_DeInit_MASK;
}


  /**
  * @brief  Initializes the OPAx peripheral according to the specified parameters
  *         in the OPA_InitStruct.
  * @note   This function is used to configure the global features of the OPA (
  *         Pos Input Signal, Filter Sampling, High Speed Mode, Polarity Reversal).
  * @param  OPAx: where x can be 0 or 1 to select the OPA peripheral.
  * @param  OPA_InitTypeDef: pointer to an OPA_InitStruct structure that contains
  *         the configuration information for the specified OPAx peripheral.
  * @retval None
  */
void OPA_Init(OPA_TypeDef* OPAx, OPA_InitTypeDef* OPA_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_OPA_ALL_PERIPH(OPAx));
	assert_param(IS_OPA_PositiveInput(OPA_InitStruct->OPA_PositiveInput));
	assert_param(IS_FUNCTIONAL_STATE(OPA_InitStruct->OPA_PowerMode));
	/*  */
	tmpreg = OPAx->CR1;
	/*  */
	tmpreg &= (u32)~OPA_CR1_MASK;
	/*  */
	tmpreg  |= 	((u32)((OPA_InitStruct->OPA_PositiveInput) |\
 				((OPA_InitStruct->OPA_PowerMode)<<7)));
	/* Write to OPAx CR1 */
	OPAx->CR1 = tmpreg;
	OPAx->CR2 |= (u32)OPA_CR2_OPAEN;
}

/**
  * @brief  Fills each OPA_InitStruct member with its default value.
  * @note   This function is used to initialize the global features of the OPA (
  *         Pos Input Signal, Filter Sampling, High Speed Mode, Polarity Reversal).
  * @param  OPA_InitStruct: pointer to an OPA_InitStruct structure which will
  *         be initialized.
  * @retval None
  */
void OPA_StructInit(OPA_InitTypeDef* OPA_InitStruct)
{
	OPA_InitStruct->OPA_PositiveInput = OPA_PositiveInput_DAC;//CMP_REFSEL_PIN;
	OPA_InitStruct->OPA_PowerMode = OPA_PowerMode_HighSpeed;
}

/**
  * @brief  Enables or disables the specified OPA peripheral.
  * @param  OPAx: where x can be 0 or 1 to select the OPA peripheral.
  * @param  NewState: new state of the OPA peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void OPA_Cmd(OPA_TypeDef* OPAx, FunctionalState NewState)   //ʹOPA
{
	/* Check the parameters */
	assert_param(IS_OPA_ALL_PERIPH(OPAx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		OPAx->CR1 |= (u32)OPA_CR1_EN;
	}
	else
	{
		OPAx->CR1 &= (u32)~((u32)OPA_CR1_EN);
	}
}



/**
  * @brief  
  * @param  
  * @param  NewState: new state of the OPA peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void OPA0_AFGndCMD(FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/*  */
		OPA0->CR2 |= (u32)OPA_CR2_AFGE;
	}
	else
	{
		/*  */
		OPA0->CR2 &= (u32)~((u32)OPA_CR2_AFGE);
	}
}


/**
  * @brief  
  * @param  
  * @param  NewState: new state of the OPA peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void OPA0_AFOutputCMD(u32 OPA0_AFOx,FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_OPA0_AFOx(OPA0_AFOx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/*  */
		OPA0->CR2 |= OPA0_AFOx;
	}
	else
	{
		/*  */
		OPA0->CR2 &= (~OPA0_AFOx);
	}
}


/**
  * @}
  */
