  /******************************************************************************
  * @file    PT32x033x_pwm.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the TIM peripheral:
  *            + TimeBase management
  *            + Output Compare management
  *            + Input Capture management
  *            + Interrupts, flags management
  *            + Clocks management
  *            + Synchronization management
  *            + Specific interface management
  *            + Specific remapping management
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_pwm.h"


/** @defgroup PWM
  * @brief PWM driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/



/**
  * @brief  Initializes the PWMx Time Base Unit peripheral according to
  *         the specified parameters in the TIM_TimeBaseInitStruct.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral
  * @note
  * @param  TimeBaseInit: pointer to a PWM_TimeBaseInitTypeDef
  *         structure that contains the configuration information for
  *         the specified PWM peripheral.
  * @retval None
  */

void PMW_TimeBaseInit(TIM1_TypeDef* PWMx, PWM_TimeBaseInitTypeDef* TimeBaseInit)
{
	u32  tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_COUNTER_RELOAD(TimeBaseInit->PWM_Period));
	assert_param(IS_PWM_CLOCKSEL(TimeBaseInit->PWM_ClockSel));
	assert_param(IS_PWM_PRESCALER(TimeBaseInit->PWM_Prescaler));
	assert_param(IS_PWM_COUNTERMODE(TimeBaseInit->PWM_CounterMode));
	assert_param(IS_PWM_COUNTERDIRRV_EN(TimeBaseInit->PWM_CounterDirRvEN));
	/* Set the counter reload value */
	PWMx->ARR = TimeBaseInit->PWM_Period;
	/* Set the PWM clcok  */
	tmpreg &= ~PWM_CR1_CLKS ;
	tmpreg |= TimeBaseInit->PWM_ClockSel;
	PWMx->CR1 = tmpreg;
	/* Set the Prescaler value */
	PWMx->PSC = TimeBaseInit->PWM_Prescaler;
	/* Select the Counter Mode */
	tmpreg = 0;
	tmpreg &= ~(PWM_CR2_DIR | PWM_CR2_CMS);
	tmpreg |=  (TimeBaseInit->PWM_CounterMode | TimeBaseInit->PWM_CounterDirRvEN);
	PWMx->CR2 = tmpreg;
}

/**
  * @brief  Initializes the PWMx channelx output
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @note
  * @param  OutInit: pointer to a PWM_OutputInitTypeDef
  *         structure that contains the configuration information for
  *         the specified PWM peripheral.
  * @retval None
  */

void PWM_OutputInit(TIM1_TypeDef* PWMx, PWM_OutputInitTypeDef* OutInit)
{
	u32 ch = 0;
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(OutInit->Channel));
	assert_param(IS_PWM_OM(OutInit->OutputMode));
	assert_param(IS_PWM_CHNCTL(OutInit->ControlN));
	assert_param(IS_PWM_IDLE(OutInit->IdleState));
	assert_param(IS_PWM_COMPAREVALUE(OutInit->CompareValue));
	assert_param(IS_PWM_DEATHTIME(OutInit->DeathTime));

	ch = OutInit->Channel;
	/* Set the CHx Compare value */
	if(ch == PWM_CH_1)	PWMx->OCR1 = OutInit->CompareValue;
	if(ch == PWM_CH_2)	PWMx->OCR2 = OutInit->CompareValue;
	if(ch == PWM_CH_3)	PWMx->OCR3 = OutInit->CompareValue;
	if(ch == PWM_CH_4)	PWMx->OCR4 = OutInit->CompareValue;
	/* Set the OCMR Reg */
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~((PWM_OCMR_OC1M << ch * 4) | (PWM_OCMR_OIS1 << ch) |  (PWM_OCMR_OC1NE << ch)) ;
	wTmpReg |=  (((OutInit->OutputMode) << ch * 4) | ((OutInit->ControlN) << ch) | ((OutInit->IdleState) << ch));
	PWMx->OCMR = wTmpReg;
	/* Set the Death time */
	PWMx->DT = OutInit->DeathTime;
}

/**
  * @brief  Initializes the PWMx channelx Capture
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @note
  * @param  CapInit: pointer to a PWM_CaptureInitTypeDef
  *         structure that contains the configuration information for
  *         the specified PWM peripheral.
  * @retval None
  */

void PWM_CaptureInit(TIM1_TypeDef* PWMx, PWM_CaptureInitTypeDef* CapInit)
{
	u32 ch = 0;
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CapInit->Channel));
	assert_param(IS_PWM_CMINT(CapInit->Capture_Interrupt));

	ch = CapInit->Channel;
	PWMx->OCMR &= ~(0x03 << ch * 4);	//ֹCHx ģʽ
	/* Set the CAPR Reg */
	wTmpReg = PWMx->CAPR;
	wTmpReg &= ~(0x0F << ch * 4);
	wTmpReg |=  (((CapInit->CaptureMode) << ch * 4) | ((CapInit->Capture_Interrupt) << ch));
	PWMx->CAPR = wTmpReg;
	
}


/**
  * @brief  Fills each TIM_TimeBaseInitStruct member with its default value.
  * @param  TIM_TimeBaseInitStruct: pointer to a TIM_TimeBaseInitTypeDef structure
  *         which will be initialized.
  * @retval None
  */
void PWM_TimeBaseStructInit(PWM_TimeBaseInitTypeDef* TimeBaseInit)
{
	/* Set the default configuration */
	TimeBaseInit->PWM_Period = 0xFFFF;
	TimeBaseInit->PWM_ClockSel = PCLK;
	TimeBaseInit->PWM_Prescaler = 0x0000;
	TimeBaseInit->PWM_CounterMode = PWM_CNT_MODE_UP;
	TimeBaseInit->PWM_CounterDirRvEN = PWM_CounterDirRv_DIS;
}

/**
  * @brief  Configures the PWMn Match Reload Times Trigger INT.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  RepeatTimes: specifies the Match Times to Trigger INT (0~0xF)
  * @note
  * @retval None
  */
void PWM_INT_RepeatTimes(TIM1_TypeDef* PWMx, uint8_t RepeatTimes)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_REPEATTIME(RepeatTimes) );
	/* Set the PWM INT_RepeatTimes */
	PWMx->ITARR = RepeatTimes;
}


/**
  * @brief  Configures the PWMx Clock Source.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  PWM_ClockSel: specifies the PWMx Clock Source
  *          This parameter can be one of the following values:
  *            @arg PCLK:
  *            @arg HCLK:
  * @note
  * @retval None
  */
void PWM_ClockSourceConfig(TIM1_TypeDef* PWMx, uint8_t PWM_ClockSel)
{
	u32  tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CLOCKSEL(PWM_ClockSel));
	/* Set the PWM Clock Source */
	tmpreg = 	PWMx->CR1;
	tmpreg &= ~PWM_CR1_CLKS ;
	tmpreg |= PWM_ClockSel;
	PWMx->CR1 = tmpreg;
}

/**
  * @brief  Configures the PWMx Prescaler.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  Prescaler: specifies the Prescaler Register value (0~255)
  * @note   The Prescaler is loaded at the PC update event or immediatly
  * @retval None
  */
void PWM_PrescalerConfig(TIM1_TypeDef* PWMx, uint8_t Prescaler)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_PRESCALER(Prescaler));
	/* Set the Prescaler value */
	PWMx->PSC = Prescaler;
}


/**
  * @brief  Configures the PWMx Reload value.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  Reload: specifies the Counter reload value (0~65535)
  * @note   The value is loaded at the update event or immediatly
  * @retval None
  */

void PWM_SetCounterReload(TIM1_TypeDef* PWMx, u16 Reload)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_COUNTER_RELOAD(Reload));
	/* Set the counter reload value */
	PWMx->ARR = Reload;
}

/**
  * @brief  Configures the PWMx Current Value.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  Counter: specifies the Counter register new value.
  * @note   The Function is valid when PWM no ouput
  * @retval None
  */
void PWM_SetCounter(TIM1_TypeDef* PWMx, u16 Counter)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_COUNTER_VALUE(Counter));
	/* Set the Counter Register value */
	PWMx->CNT = Counter;
}

/**
  * @brief  Gets the PWMx Counter value.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @retval Counter Register value.
  */
u16 PWM_GetCounter(TIM1_TypeDef* PWMx)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	/* Get the Counter Register value */
	return  PWMx->CNT;
}

/**
  * @brief  Gets the PWMx Current INT RepeatTimes.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @retval Current INT RepeatTimes.
  */
uint8_t PWM_GetINT_RepeatTimes(TIM1_TypeDef* PWMx)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	/* Get the Counter Register value */
	return  PWMx->ITCNT;
}


/**
  * @brief  Gets the PWMx Prescaler value.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @retval Prescaler Register value.
  */
u16 PWM_GetPrescaler(TIM1_TypeDef* PWMx)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	/* Get the Prescaler Register value */
	return  PWMx->PSC;
}

/**
  * @brief  Specifies the PWMx Counter Mode to be used.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @note
  * @param  CounterMode: specifies the Counter Mode to be used
  *          This parameter can be one of the following values:
  *            @arg PWM_CNT_MODE_UP: TIM Up Counting Mode
  *            @arg PWM_CNT_MODE_DOWN: TIM Down Counting Mode
  * @param  DirRv_EN: specifies the Counter Direction Reverse
  *            @arg PWM_CounterDirRv_EN: TIM Direction Reverse Enable
  *            @arg PWM_CounterDirRv_DIS: TIM Direction Reverse Disable
  * @retval None
  */
void PWM_CounterModeConfig(TIM1_TypeDef* PWMx, uint8_t CounterMode, uint8_t DirRv_EN)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_COUNTERMODE(CounterMode));
	assert_param(IS_PWM_COUNTERDIRRV_EN(DirRv_EN));
	/* Select the Counter Mode */
	tmpreg = PWMx->CR2;
	tmpreg &= ~(PWM_CR2_DIR | PWM_CR2_CMS);
	tmpreg |=  (CounterMode | DirRv_EN);
	PWMx->CR2 = tmpreg;
}

/**
  * @brief  Update settiongs of the PWMx
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @retval None
  */

void PWM_UpdateCmd(TIM1_TypeDef* PWMx)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	/* Set the update bit */
	PWMx->CR1 |= PWM_CR1_UG;
}

/**
  * @brief  Enable or Disable PWMx Counter
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */

void PWM_Cmd(TIM1_TypeDef* PWMx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState == ENABLE)
	{
		/* Set the update bit */
		PWMx->CR1 |= PWM_CR1_EN;
	}
	else
	{
		/* Reset the update bit */
		PWMx->CR1 &= (~PWM_CR1_EN);
	}
}


void PWM_BreakInput_Cmd(TIM1_TypeDef* PWMx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable BreakInput*/
		PWMx->OCMR |= PWM_OCMR_BKE;
	}
	else
	{
		/* Disable BreakInput*/
		PWMx->OCMR &= ~PWM_OCMR_BKE;
	}
}

void PWM_BKI_LevelConfig(TIM1_TypeDef* PWMx, u32 BKI_Level)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_BIK_LEVEL(PWM_BKI_HIGHLEVEL));
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~PWM_OCMR_BKP;
	wTmpReg |= BKI_Level;
	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Specifies the PWMx Counter Mode to be used.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  PWM_INT: Specify the PWM interrupts sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *        @arg PWM_INT_MR0      : PWM MR0 Reload value match interrupt
  *        @arg PWM_INT_MR1      : PWM CH1 MR1 vaule match interrupt
  *        @arg PWM_INT_MR2      : PWM CH2 MR2 vaule match interrupt
  *        @arg PWM_INT_MR3      : PWM CH3 MR3 vaule match interrupt
  *        @arg PWM_INT_MR4      : PWM CH4 MR4 vaule match interrupt
  *        @arg PWM_INT_CAP1     : PWM CH1 Capture interrupt
  *        @arg PWM_INT_CAP2     : PWM CH2 Capture interrupt
  *        @arg PWM_INT_CAP3     : PWM CH3 Capture interrupt
  *        @arg PWM_INT_CAP4     : PWM CH4 Capture interrupt
  *        @arg PWM_INT_BK       : PWM break interrupt
  * @param  NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */

void PWM_IntConfig(TIM1_TypeDef* PWMx, u32 PWM_INT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_PWM_INT(PWM_INT));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));
	
	if((PWM_INT==PWM_INT_MR0) | (PWM_INT==PWM_INT_MR1) | (PWM_INT==PWM_INT_MR2) | (PWM_INT==PWM_INT_MR3)| (PWM_INT==PWM_INT_MR4))
	{
    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      PWMx->CR2 |= PWM_INT;
    }
    else
    {
      /* Disable the interrupt sources */
      PWMx->CR2 &= ~PWM_INT;
    }
  }
	
	if(PWM_INT==PWM_INT_BK)
	{
    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      PWMx->DT |= PWM_INT;
    }
    else
    {
      /* Disable the interrupt sources */
      PWMx->DT &= ~PWM_INT;
    }
  }	
	
}

/**
  * @brief  Specifies the PWMx Counter Mode to be used.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  PWM_INT: Specify the PWM interrupts sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *        @arg PWM_INT_CAP1     : PWM CH1 Capture interrupt
  *        @arg PWM_INT_CAP2     : PWM CH2 Capture interrupt
  *        @arg PWM_INT_CAP3     : PWM CH3 Capture interrupt
  *        @arg PWM_INT_CAP4     : PWM CH4 Capture interrupt
  * @param  NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void PWM_CapITConfig(TIM1_TypeDef* PWMx,u32 PWM_CAP_INT,FunctionalState NewState)
{
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_CAP_INT(PWM_CAP_INT));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));
	
	
	if((PWM_CAP_INT==PWM_INT_CAP1) | (PWM_CAP_INT==PWM_INT_CAP2) | (PWM_CAP_INT==PWM_INT_CAP3) | (PWM_CAP_INT==PWM_INT_CAP4))
	{
    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      PWMx->CAPR |= PWM_CAP_INT;
    }
    else
    {
      /* Disable the interrupt sources */
      PWMx->CAPR &= ~PWM_CAP_INT;
    }
  }
}

/**
  * @brief  Specifies the PWMx Counter Mode to be used.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  PWM_FLAG: Specify the flag to be checked.
  *         This parameter can be one of the following values:
  *         @arg PWM_FLAG_MR0       : PWM MR0 Reload value match flag
  *         @arg PWM_FLAG_MR1       : PWM CH1 MR1 vaule match flag
  *         @arg PWM_FLAG_MR2       : PWM CH2 MR2 vaule match flag
  *         @arg PWM_FLAG_MR3       : PWM CH3 MR3 vaule match flag
  *         @arg PWM_FLAG_MR4       : PWM CH4 MR4 vaule match flag
  *         @arg PWM_FLAG_CAP1_RE   : PWM CH1 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP1_FE   : PWM CH1 Capture Falling Edge flag
  *         @arg PWM_FLAG_CAP2_RE   : PWM CH2 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP2_FE   : PWM CH2 Capture Falling Edge flag
  *         @arg PWM_FLAG_CAP3_RE   : PWM CH3 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP3_FE   : PWM CH3 Capture Falling Edge flag
  *         @arg PWM_FLAG_CAP4_RE   : PWM CH4 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP4_FE   : PWM CH4 Capture Falling Edge flag
  *         @arg PWM_FLAG_BK        : PWM break flag
  * @return FlagStatus of PWM_FLAG (SET or RESET).
  */
FlagStatus PWM_GetFlagStatus(TIM1_TypeDef* PWMx, u32 PWM_FLAG)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_PWM_FLAG(PWM_FLAG));
	if ((PWMx->SR & PWM_FLAG) != 0)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/**
  * @brief  Specifies the PWMx Counter Mode to be used.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  PWM_FLAG: Specify the flag to be cleared.
  *         This parameter can be one of the following values:
  *         @arg PWM_FLAG_MR0       : PWM MR0 Reload value match flag
  *         @arg PWM_FLAG_MR1       : PWM CH1 MR1 vaule match flag
  *         @arg PWM_FLAG_MR2       : PWM CH2 MR2 vaule match flag
  *         @arg PWM_FLAG_MR3       : PWM CH3 MR3 vaule match flag
  *         @arg PWM_FLAG_MR4       : PWM CH4 MR4 vaule match flag
  *         @arg PWM_FLAG_CAP1_RE   : PWM CH1 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP1_FE   : PWM CH1 Capture Falling Edge flag
  *         @arg PWM_FLAG_CAP2_RE   : PWM CH2 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP2_FE   : PWM CH2 Capture Falling Edge flag
  *         @arg PWM_FLAG_CAP3_RE   : PWM CH3 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP3_FE   : PWM CH3 Capture Falling Edge flag
  *         @arg PWM_FLAG_CAP4_RE   : PWM CH4 Capture Rising  Edge flag
  *         @arg PWM_FLAG_CAP4_FE   : PWM CH4 Capture Falling Edge flag
  *         @arg PWM_FLAG_BK        : PWM break flag
  * @retval None
  */
void PWM_ClearFlag(TIM1_TypeDef* PWMx, u32 PWM_FLAG)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CLEAR_FLAG(PWM_FLAG));
	/* Clear the flags */
	PWMx->SR = PWM_FLAG;
}


/**
  * @brief  Enable or Disable PWMx CHx RisingEdge Capture
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1       :
  *         @arg PWM_CH_2       :
  *         @arg PWM_CH_3       :
  *         @arg PWM_CH_4       :
  * @param  NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void PWM_RE_CaptureConfig(TIM1_TypeDef* PWMx, uint8_t CHx, FunctionalState NewState)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	wTmpReg = PWMx->CAPR;
	wTmpReg &= ~(PWM_CAPR_IC1R << CHx * 4);
	if (NewState != DISABLE)
	{
		/* Enable RisingEdge Capture*/
		wTmpReg |= (PWM_CAPR_IC1R << CHx * 4);
	}
	else
	{
		/* Disable RisingEdge Capture*/
		wTmpReg |= 0;
	}
	PWMx->CAPR = wTmpReg;
}


/**
  * @brief  Enable or Disable PWMx CHx FallingEdge Capture
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1       :
  *         @arg PWM_CH_2       :
  *         @arg PWM_CH_3       :
  *         @arg PWM_CH_4       :
  * @param  NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void PWM_FE_CaptureConfig(TIM1_TypeDef* PWMx, uint8_t CHx, FunctionalState NewState)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	wTmpReg = PWMx->CAPR;
	wTmpReg &= ~(PWM_CAPR_IC1F << CHx * 4);
	if (NewState != DISABLE)
	{
		/* Enable FallingEdge Capture*/
		wTmpReg |= (PWM_CAPR_IC1F << CHx * 4);
	}
	else
	{
		/* Disable FallingEdge Capture*/
		wTmpReg |= 0;
	}
	PWMx->CAPR = wTmpReg;
}

/**
  * @brief  Enable or Disable PWMx CHx Capture Rester
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1       :
  *         @arg PWM_CH_2       :
  *         @arg PWM_CH_3       :
  *         @arg PWM_CH_4       :
  * @param  NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void PWM_RST_CaptureConfig(TIM1_TypeDef* PWMx, uint8_t CHx, FunctionalState NewState)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	wTmpReg = PWMx->CAPR;
	wTmpReg &= ~(PWM_CAPR_IC1RC << CHx * 4);
	if (NewState != DISABLE)
	{
		/* Enable Capture Reset*/
		wTmpReg |= (PWM_CAPR_IC1RC << CHx * 4);
	}
	else
	{
		/* Disable Capture Reset*/
		wTmpReg |= 0;
	}
	PWMx->CAPR = wTmpReg;
}


/**
  * @brief  Gets the PWMx CHx Capture Counter value.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1       :
  *         @arg PWM_CH_2       :
  *         @arg PWM_CH_3       :
  *         @arg PWM_CH_4       :
  * @retval Counter Register value.
  */
u16 PWM_GetCaptureCounter(TIM1_TypeDef* PWMx, uint8_t CHx)
{
	u16 capture_cnt = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	/* Get the Capture Counter Register value */
	if(CHx == PWM_CH_1)
		capture_cnt = PWMx->ICR1 ;
	if(CHx == PWM_CH_2)
		capture_cnt = PWMx->ICR2 ;
	if(CHx == PWM_CH_3)
		capture_cnt = PWMx->ICR3 ;
	if(CHx == PWM_CH_4)
		capture_cnt = PWMx->ICR4 ;
	return  capture_cnt;
}

/**
  * @brief  Gets the PWMx CHx Capture Edge .
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1       :
  *         @arg PWM_CH_2       :
  *         @arg PWM_CH_3       :
  *         @arg PWM_CH_4       :
  * @retval  Capture Edge.
  *         This parameter can be one of the following values:
  *         @arg CAPTURE_FE     :  capture falling edge
  *         @arg CAPTURE_RE     :  capture rising  edge
  */
u16 PWM_GetCaptureEdge(TIM1_TypeDef* PWMx, uint8_t CHx)
{
	u16 capture_edge = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	/* Get the Capture Counter Register value */
	if(CHx == PWM_CH_1)
		capture_edge = (PWMx->ICR1 >> 16);
	if(CHx == PWM_CH_2)
		capture_edge = (PWMx->ICR2 >> 16) ;
	if(CHx == PWM_CH_3)
		capture_edge = (PWMx->ICR3 >> 16) ;
	if(CHx == PWM_CH_4)
		capture_edge = (PWMx->ICR4 >> 16) ;
	return  capture_edge;
}


/**
  * @brief  Set the PWMx CHx Output Mode Config
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1        :
  *         @arg PWM_CH_2        :
  *         @arg PWM_CH_3        :
  *         @arg PWM_CH_4        :
  * @param  Mode: Specify the Channel output mode
  *         This parameter can be one of the following values:
  *         @arg NO_OUTPUT       :  PWM channel no output
  *         @arg COMPAER_MODE1   :  PWM channel output high when tc <= compare value,output low when tc > compare value
  *         @arg COMPAER_MODE2   :  PWM channel output low when tc <= compare value,output high when tc > compare value
  *         @arg COMPAER_MODE3   :  PWM channel output reverse when tc match compare value
  * @retval None
  */
void PWM_OD_Config(TIM1_TypeDef* PWMx, uint8_t CHx, u32 Mode)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_PWM_OM(Mode));
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~(PWM_OCMR_OC1M << CHx * 4);
	wTmpReg |= (Mode << CHx * 4);
	/* Set PWMx CH Output Mode*/
	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Set the PWMx CHx Output idle
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1        :
  *         @arg PWM_CH_2        :
  *         @arg PWM_CH_3        :
  *         @arg PWM_CH_4        :
  * @param  Mode: Specify the Channel output mode
  *         This parameter can be one of the following values:
  *         @arg PWM_IDLE_LOW    :  PWM channel idle is low
  *         @arg PWM_IDLE_HIGH   :  PWM channel idle is high
  * @retval None
  */
void PWM_IDLE_Config(TIM1_TypeDef* PWMx, uint8_t CHx, u32 Idle)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_PWM_IDLE(Idle));
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~(PWM_OCMR_OIS1 << CHx);
	wTmpReg |= (Idle << CHx);
	/* Set PWMx CH Output Idle*/
	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Enable or Disable the PWMx CHx Complementary Output
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1        :
  *         @arg PWM_CH_2        :
  *         @arg PWM_CH_3        :
  *         @arg PWM_CH_4        :
  * @param  CHNCTL: Specify the Channel output mode
  *         This parameter can be one of the following values:
  *         @arg PWM_CHNCTL_DISABLE  :  Disable PWMx CHx Complementary Output
  *         @arg PWM_CHNCTL_ENABLE   :  Enable PWMx CHx Complementary Output
  * @retval None
  */
void PWM_CHN_Config(TIM1_TypeDef* PWMx, uint8_t CHx, u32 CHNCTL)
{
	u32 wTmpReg = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_PWM_CHNCTL(CHNCTL));
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~(PWM_OCMR_OC1NE << CHx);
	wTmpReg |= (CHNCTL << CHx);
	/* Set PWMx CH Complementary Output*/
	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Set the PWMx Death Time
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  DeathTime: Specify the Death value (0~0xFF)
  * @retval None
  */
void PWM_DT_Config(TIM1_TypeDef* PWMx, uint8_t CHx, uint8_t DeathTime)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_DEATHTIME(DeathTime));
	/* Set PWMx Death Time*/
	PWMx->DT = DeathTime;
}

/**
  * @brief  Set the PWMx CHx Compare Value
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
  * @param  CHx:  Specify the Channel to be selected
  *         This parameter can be one of the following values:
  *         @arg PWM_CH_1        :
  *         @arg PWM_CH_2        :
  *         @arg PWM_CH_3        :
  *         @arg PWM_CH_4        :
  * @param  Cmp: Specify the CHx Output Compare Value value (0~0xFFFF)
  * @retval None
  */
void PWM_SetCompare(TIM1_TypeDef* PWMx, uint8_t CHx, u16 Cmp)
{
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_CHCHANNEL(CHx));
	assert_param(IS_PWM_COMPAREVALUE(Cmp));
	/* Set PWM CH Output Compare Value*/
	if(CHx == PWM_CH_1)
		PWMx->OCR1 = Cmp;
	if(CHx == PWM_CH_2)
		PWMx->OCR2 = Cmp;
	if(CHx == PWM_CH_3)
		PWMx->OCR3 = Cmp;
	if(CHx == PWM_CH_4)
		PWMx->OCR4 = Cmp;
}

void PWM_DeInit(TIM1_TypeDef* PWMx)
{
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	PWMx->SR=0x3fff;
	PWMx->CR1=0x0;
	PWMx->ITARR=0x0;
	PWMx->ITCNT=0x0;
	PWMx->PSC=0x0;
	PWMx->CNT=0x0;
	PWMx->CR2=0x0;
	PWMx->ARR=0x0;
	PWMx->OCR1=0x0;
	PWMx->OCR2=0x0;
	PWMx->OCR3=0x0;
	PWMx->OCR4=0x0;
	PWMx->CAPR=0x0;
	PWMx->OCMR=0x0;
	PWMx->DT=0x0;
}
/**
  * @}
  */
