/******************************************************************************
  * @file    PT32x033x_pwr.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage Power Controller (PWR) peripheral
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_pwr.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/* CR register bit mask */
#define CR_PLS_MASK              ((u32)0x00000000)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/




/**
  * @brief  Configures the voltage threshold detected by the Power Voltage Detector(PVD).
  * @param  PWR_PVDLevel: specifies the PVD detection level
  *          This parameter can be one of the following values:
  *             @arg PWR_PVDLevel_0
  *             @arg PWR_PVDLevel_1
  *             @arg PWR_PVDLevel_2
  *             @arg PWR_PVDLevel_3
  *             @arg PWR_PVDLevel_4
  *             @arg PWR_PVDLevel_5
  *             @arg PWR_PVDLevel_6
  *             @arg PWR_PVDLevel_7
  * @note   Refer to the electrical characteristics of your device datasheet for
  *         more details about the voltage threshold corresponding to each
  *         detection level.
  * @retval None
  */
void PWR_PVDLevelConfig(u32 PWR_PVDLevel)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_PWR_PVD_LEVEL(PWR_PVDLevel));
	tmpreg = PWR->PVDR;
	/* Clear PLS[7:5] bits */
	tmpreg &= CR_PLS_MASK;
	/* Set PLS[7:5] bits according to PWR_PVDLevel value */
	tmpreg |= PWR_PVDLevel;
	/* Store the new value */
	PWR->PVDR = tmpreg;
}


/**
  * @brief  Enables or disables the Power Voltage Detector(PVD).
  * @param  NewState: new state of the PVD.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void PWR_PVDCmd(FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the PVD */
		PWR->PVDR |= PWR_PVDR_EN;
	}
	else
	{
		/* Disable the PVD */
		PWR->PVDR &= (u32)~((u32)PWR_PVDR_EN);
	}
}



/**
  * @brief  Enables or disables the BG2V.
  * @param  NewState: new state of the BG2V.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void PWR_BG2VCmd(FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the PVD */
		PWR->VRFR |= PWR_VRFR_EN;
	}
	else
	{
		/* Disable the PVD */
		PWR->VRFR &= (u32)~((u32)PWR_VRFR_EN);
	}
}



/**
  * @brief  Enters Sleep mode.
  * @note   In Sleep mode, all I/O pins keep the same state as in Run mode.
  * @param  PWR_SLEEPEntry: specifies if SLEEP mode in entered with WFI or WFE instruction.
  *          This parameter can be one of the following values:
  *             @arg PWR_SLEEPEntry_WFI: enter SLEEP mode with WFI instruction
  *             @arg PWR_SLEEPEntry_WFE: enter SLEEP mode with WFE instruction
  * @retval None
  */
void PWR_EnterSleepMode(uint8_t PWR_SLEEPEntry)
{
	/* Check the parameters */
	assert_param(IS_PWR_SLEEP_ENTRY(PWR_SLEEPEntry));
	/* Clear SLEEPDEEP bit of Cortex-M0 System Control Register */
	SCB->SCR &= (u32)~((u32)SCB_SCR_SLEEPDEEP_Msk);
	/* Select SLEEP mode entry -------------------------------------------------*/
	if(PWR_SLEEPEntry == PWR_SLEEPEntry_WFI)
	{
		/* Request Wait For Interrupt */
		__WFI();
	}
	else
	{
		/* Request Wait For Event */
		__SEV();
		__WFE();
		__WFE();
	}
}



/**
  * @brief  Enters STANDBY mode.
  * @note   In Standby mode, all I/O pins are high impedance except for:
  *          - Reset pad (still available)
  *          - RTC_AF1 pin (PC13) if configured for Wakeup pin 2 (WKUP2), tamper,
  *             time-stamp, RTC Alarm out, or RTC clock calibration out.
  *          - WKUP pin 1 (PA0) if enabled.
  * @note The Wakeup flag (WUF) need to be cleared at application level before to call this function
  *             @arg PWR_STANDBYEntry_WFI: enter DEEPSLEEP mode with WFI instruction
  *             @arg PWR_STANDBYEntry_WFE: enter DEEPSLEEP mode with WFE instruction
  * @retval None
  */
void PWR_EnterSTANDBYMode(uint8_t PWR_DeepStandbyEntry)
{
	/* Check the parameters */
	assert_param(IS_PWR_STANDBY_ENTRY(PWR_DeepStandbyEntry));
	/* Set SLEEPDEEP bit of Cortex-M0 System Control Register */
	SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
	/* Request Wait For Interrupt */
	/* Select SLEEP mode entry -------------------------------------------------*/
	if(PWR_DeepStandbyEntry == PWR_STANDBYEntry_WFI)
	{
		/* Request Wait For Interrupt */
		__WFI();
	}
	else
	{
		/* Request Wait For Event */
		__WFE();
		__WFE();
	}
}





/************************ (C) *****END OF FILE****/
