  /******************************************************************************
  * @file    PT32x033x_rcc.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the SYS peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *            
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_rcc.h"

#define  HSI   8000000UL
#define  LSI   32000UL


/** @defgroup RCC
  * @brief RCC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/**
  * @brief  Gets the SYS_CLK value.
  * @param  None
  * @retval SYS_CLK value
  */
u32 RCC_GetSYSCLKFreq(void)
{
	u32 SYSCLK=clk_get_sysclk();
	return SYSCLK;
}

/**
  * @brief  Gets the HCLK_CLK value.
  * @param  None
  * @retval HCLK_CLK value
  */
u32 RCC_GetHCLKFreq(void)
{
    u32 HCLK_Freq = 0;
    u32 HCLK_DIV = 0;
		u32 SYSCLK = 0;
    SYSCLK = RCC_GetSYSCLKFreq();
    HCLK_DIV = ((RCC->CFGR & RCC_CFGR_HPRE) >> 4) ;
	  HCLK_Freq = SYSCLK >> HCLK_DIV;
    return HCLK_Freq;
}

/**
  * @brief  Gets the PCLK_CLK value.
  * @param  None
  * @retval PCLK_CLK value
  */
u32 RCC_GetPCLKFreq(void)
{
    u32 PCLK_Freq = 0;
    u32 PCLK_DIV = 0;
    
    u32 HCLK_Freq = RCC_GetHCLKFreq();
    PCLK_DIV = ((RCC->CFGR & RCC_CFGR_PPRE) >> 8) ;
    PCLK_Freq = HCLK_Freq >> PCLK_DIV;
    
    return PCLK_Freq;
}


/**
  * @brief  Check whether the specific global reset flag is set or not.
 *  @param RSTCU_RSTF: specify the reset flag.
 *        This parameter can be:
 *        @arg RCC_RSR_SFR
 *        @arg RCC_RSR_IWDGR
 *        @arg RCC_RSR_POR
 *        @arg RCC_RSR_PINR
 *  @retval SET or RESET
  */
FlagStatus RCC_GetResetFlagStatus(u32 RCC_RSTF)
{
	u32 tmp;
	/* Check the parameters */
	assert_param(IS_RSTCU_FLAG(RCC_RSTF));
	tmp = (RCC->RSR & (RCC_RSTF));
	if (tmp != RESET)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/**
 * @brief  Clear the specific global reset flag.
 * @param RSTCU_RSTF: specify the reset flag.
 *        This parameter can be:
 *        @arg RCC_RSR_SFR
 *        @arg RCC_RSR_IWDGR
 *        @arg RCC_RSR_POR
 *        @arg RCC_RSR_PINR
 * @retval None
  */
void RCC_ClearResetFlag(u32 RSTCU_RSTF)
{
	/* Check the parameters */
	assert_param(IS_RSTCU_FLAG(RSTCU_RSTF));
	RCC->RSR |= RSTCU_RSTF; 
}


/**
  * @brief  Enables or disables the specified Reset Source.
  * @param  RstSource: specifies the Reset source to be enabled or disabled.
  *          This parameter can be one of the following values:
 *        @arg RCC_RSR_SFR
 *        @arg RCC_RSR_IWDGR
 *        @arg RCC_RSR_POR
 *        @arg RCC_RSR_PINR
  * @param  NewState: new state of the specified RSTEN SOURCE
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RCC_RestConfig(u32 RstSource, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_RSTEN_SOURCE(RstSource));
	if (NewState != DISABLE)
	{
		RCC->RCR |= RstSource;
	}
	else
	{
		RCC->RCR &= (u16)~RstSource;
	}
}

/**
  * @brief  Enables or disables the specified OSC
  * @param  OSC_Source: specifies the OSC Source to be enabled or disabled.
  *         This parameter can be one of the following values:
  *         @arg Oscillator_HSE	: Specify the new state of Extern High Crystal 
  *         @arg Oscillator_LSE	: Specify the new state of Extern Low Crystal 
  *         @arg Oscillator_HSI	: Specify the new state of Internal High Crystal 
  *         @arg Oscillator_LSI	: Specify the new state of Internal Low Crystal 
  *         @return None
  */
void RCC_OscillatorCmd(u32 OSC_Source,FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_RCC_Osc_Source(OSC_Source));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	
	if (NewState != DISABLE)
	{
		if(OSC_Source==Oscillator_HSE)		RCC->HSECR1 |=0x1;
		if(OSC_Source==Oscillator_LSE)		RCC->LSECR1 |= 0X01;
		if(OSC_Source==Oscillator_HSI)		RCC->HSICR |=0x1;
		if(OSC_Source==Oscillator_LSI)		RCC->LSICR |= 0X01;
	}
	else
	{
		if(OSC_Source==Oscillator_HSE)		RCC->HSECR1 &=0xFFFFFFFE;
		if(OSC_Source==Oscillator_LSE)		RCC->LSECR1 &=0xFFFFFFFE;
		if(OSC_Source==Oscillator_HSI)		RCC->HSICR &=0xFFFFFFFE;
		if(OSC_Source==Oscillator_LSI)		RCC->LSICR &=0xFFFFFFFE;
	}
}

/**
  * @brief  Determine whether Crystal is stable 
  * @param  OSC_FLAG: Specify the flag to be checked.
  *         This parameter can be one of the following values:
  *         @arg HCK_FLAG_RDY       : Determine whether High Crystal is stable 
  *         @arg LCK_FLAG_RDY       : Determine whether Low Crystal is stable 
  *         @return FlagStatus of OSC_FLAG (SET or RESET).
  */
FlagStatus RCC_OscillatorReady(u32 OSC_FLAG)
{
	u8 Addr_offset=0;
	/* Check the parameters */
	assert_param(IS_RCC_OscRdy_FLAG(OSC_FLAG));
	if(OSC_FLAG==HCK_FLAG_RDY)		Addr_offset=0X0;
	if(OSC_FLAG==LCK_FLAG_RDY)		Addr_offset=0X20;

	if((*((u32 volatile *) (RCC_BASE + Addr_offset)) & RCC_HSECR1_RDY) == RCC_HSECR1_RDY)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

  /**
  * @brief  Enables or disables the specified FD State.
  * @param  NewState: new state of the FD State.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RCC_PLL_Cmd(FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if (NewState != DISABLE)
	{
		RCC->PCR |= 0x1;
	}
	else
	{
		RCC->PCR &=0xFFFFFFFE;
	}
}

/**
  * @brief  Reset Remap Bit.
  * @param  None
  * @retval None
  */
void RCC_Retriming(void)
{
	RCC->HSFRR = RCC_HSFRR_KEY;
}

/*********************************************************************************************************//**
 * @brief Clear all of the global reset flag.
 * @retval None
 ************************************************************************************************************/
void RCC_ClearAllResetFlag(void)
{
	RCC->RSR = (u32)0xFF ;   /* Write 1 to clear */
}

/**
* @brief  LSE Capacitance Raw Adjustment.
* @param  value: select raw capacitance of LSE.
*          This parameter can be: 0-15.
* @retval None
*/
void RCC_LSECapRawAdjust(u8 value)
{
	/* Check the parameters */
	assert_param(IS_LSECAP_RAWADJUST(value));
	RCC->LSECR1&=~(RCC_LSECR1_CRA);
	RCC->LSECR1|=(u32)value<<16;
}
/**
* @brief  LSE Capacitance Fine Adjustment.
* @param  value: select raw capacitance of LSE.
*          This parameter can be: 0-15.
* @retval None
*/
void RCC_LSECapFineAdjust(u8 value)
{
	assert_param(IS_LSECAP_FINEADJUST(value));
	RCC->LSECR1&=~(RCC_LSECR1_CFA);
	RCC->LSECR1|=(u32)value<<20;	
}
/**
* @brief  LSE capacitance select.
* @param  value: select capacitance of LSE.
*          This parameter can be: 0-15.
* @retval None
*/
void RCC_LSECapSelect(u8 value)
{
	assert_param(IS_LSECAP_SELECT(value));
	RCC->LSECR1&=~(RCC_LSECR1_CS);
	RCC->LSECR1|=(u32)value<<24;	
}
/**
  * @}
  */

