  /******************************************************************************
  * @file    PT32x033x_rtc.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the RTC peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *           + Value Type management
  *            
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

  
/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_rtc.h"

/** @defgroup RTC
  * @brief RTC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define RTC_DeInit_MASK			((u32)0x00000000)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

void RTC_DeInit(void)
{
	RTC->CR = RTC_DeInit_MASK;
}

void RTC_Init(RTC_InitTypeDef* RTC_InitStruct)
{
	u32 tmpreg = 0;
	assert_param(IS_RTC_TF_SET(RTC_InitStruct->RTC_TimeFormat));
	assert_param(IS_RTC_CLK_SET(RTC_InitStruct->RTC_ClkSource));
	assert_param(IS_RTC_BEACON_SET(RTC_InitStruct->RTC_BeaconOut));
	assert_param(IS_RTC_TME_SET(RTC_InitStruct->RTC_TrimMode));
	tmpreg = RTC->CR;
	tmpreg &= 0xFFFFFFE1;
	tmpreg  |=	RTC_InitStruct->RTC_TimeFormat|
				RTC_InitStruct->RTC_ClkSource|
				RTC_InitStruct->RTC_BeaconOut|
				RTC_InitStruct->RTC_TrimMode;
	RTC->CR=tmpreg;
}

void RTC_StructInit(RTC_InitTypeDef* RTC_InitStruct)
{
	RTC_InitStruct->RTC_TimeFormat=RTC_TimeFormat_24;
	RTC_InitStruct->RTC_ClkSource = RTC_ClkSource_LSI;
	RTC_InitStruct->RTC_BeaconOut = RTC_BeaconOut_DISABLE;
	RTC_InitStruct->RTC_TrimMode=RTC_TrimMode_DISABLE;
}


/**
  * @brief  Enables or disables the specified RTC interrupts.
  * @param  RTC_IT: specifies the RTC interrupts sources to be enabled or disabled.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_IT_CSEC: Second interrupt
  *     @arg RTC_IT_CMIN: Minutes interrupt
  *     @arg RTC_IT_CHOUR: Hour interrupt
  *     @arg RTC_IT_CDAY: DAY interrupt
  *     @arg RTC_IT_CMON: Month interrupt
  *     @arg RTC_IT_ALM: Alarm interrupt
  * @param  NewState: new state of the specified RTC interrupts.
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RTC_ITConfig(u16 RTC_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_RTC_SET_IT(RTC_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		RTC->IE |= RTC_IT;
	}
	else
	{
		RTC->IE &= (u16)~RTC_IT;
	}
}


/**
  * @brief  Checks whether the specified RTC interrupt has occurred or not.
  * @param  RTC_IT: specifies the RTC interrupts sources to check.
  *   This parameter can be one of the following values:
  *     @arg RTC_IT_CSEC: Second interrupt
  *     @arg RTC_IT_CMIN: Minutes interrupt
  *     @arg RTC_IT_CHOUR: Hour interrupt
  *     @arg RTC_IT_CDAY: DAY interrupt
  *     @arg RTC_IT_CMON: Month interrupt
  *     @arg RTC_IT_ALM: Alarm interrupt
  * @retval The new state of the RTC_IT (SET or RESET).
  */
ITStatus RTC_GetITStatus(u16 RTC_IT)
{
	ITStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_RTC_GET_IT(RTC_IT));
	if ((RTC->IE & RTC_IT) != (u16)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return bitstatus;
}


/**
  * @brief  Clears the RTC's pending flags.
  * @param  RTC_FLAG: specifies the flag to clear.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_FLAG_CSEC
  *     @arg RTC_FLAG_CMIN
  *     @arg RTC_FLAG_CHOUR
  *     @arg RTC_FLAG_CDAY
  *     @arg RTC_FLAG_CMON
  *     @arg RTC_FLAG_ALM
  
  * @retval None
  */
void RTC_ClearFlag(u16 RTC_FLAG)
{
	/* Check the parameters */
	assert_param(IS_RTC_CLEAR_FLAG(RTC_FLAG));
	/* Clear the corresponding RTC flag */
	RTC->SR2 = RTC_FLAG;
}

/**
  * @brief  Checks whether the specified RTC flag is set or not.
  * @param  RTC_FLAG: specifies the flag to check.
  *   This parameter can be one the following values:
  *     @arg RTC_FLAG_CSEC
  *     @arg RTC_FLAG_CMIN
  *     @arg RTC_FLAG_CHOUR
  *     @arg RTC_FLAG_CDAY
  *     @arg RTC_FLAG_CMON
  *     @arg RTC_FLAG_ALM
  *     @arg RTC_FLAG_UnLock
  *     @arg RTC_FLAG_Busy
  *     @arg RTC_FLAG_Lock
  *     @arg RTC_FLAG_UnLockSync
  *     @arg RTC_FLAG_LockSync
  * @retval The new state of RTC_FLAG (SET or RESET).
  */
FlagStatus RTC_GetFlagStatus(u16 RTC_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_RTC_GET_FLAG(RTC_FLAG));
	if(RTC_FLAG<=RTC_FLAG_ALM)
	{
		if ((RTC->SR2 & RTC_FLAG) != (u16)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	else if(RTC_FLAG<=RTC_FLAG_Busy)
	{
		RTC_FLAG>>=6;
		if ((RTC->SR1 & RTC_FLAG) != (u16)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	else
	{
		RTC_FLAG>>=8;
		if ((RTC->LOCK & RTC_FLAG) != (u16)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	return bitstatus;
}


/**
  * @brief 
  * @param 
  * @retval
  */
void RTC_UnlockCMD(FunctionalState NewState)
{
	if(NewState==ENABLE)
	{
		RTC->LOCK = 0x1acce551;
		while(RTC_GetFlagStatus(RTC_FLAG_UnLockSync)==SET);
	}
	else
	{
		RTC->LOCK = 0x0;
		while(RTC_GetFlagStatus(RTC_FLAG_LockSync)==SET);
	}
}


/**
  * @brief Convert BCD code to decimal
  * @param  Value :BCD code
  * @retval decimal Value
  */
u8 BCD_ToDEC(u8 Value)
{
	u8 temp;
	temp = ((Value>>4)*10 + (Value&0x0f));
	return temp;
}

/**
  * @brief Convert decimal to  BCD code 
  * @param  Value :decimal Value
  * @retval BCD code
  */
u8 DEC_ToBCD(u8 Value)
{
	u8 temp;
	temp = (((Value/10)<<4) + (Value%10));
	return temp;
}

/**
  * @brief Get Current Date and Time value 
  * @param  pDt :struct of date time
  * @retval BCD code
  */
void RTC_GetCalendarTime(RTC_Calendar * pDt)
{
	pDt->TimeFormat = (u8)((RTC->CR >> 1) & 0x01);	//0: 24-Hour,1: 12-Hour
	pDt->Week = (u8)RTC->CWEEK;			//(1~7) 1:monday,...,7 sunday
	pDt->Year = (u8)RTC->CYEAR;
	pDt->Month = (u8)RTC->CMONTH;
	pDt->Day = (u8)RTC->CDAY;
	pDt->Hour = (u8)RTC->CHOUR;
	pDt->Minute = (u8)RTC->CMIN;
	pDt->Second = (u8)RTC->CSEC;
	pDt->PM = (u32)(RTC->CHOUR&RTC_CHOUR_PM)>>8;
}

/**
  * @brief Set Current Date and Time  
  * @param  pDt :struct of date time -- dec format
  * @retval BCD code
  */
void RTC_SetCalendarTime(RTC_Calendar * pDt)
{
	assert_param(pDt->Year < 99);
	assert_param(pDt->Month < 13);
	assert_param(pDt->Day < 32);
	assert_param(pDt->Hour < 24);
	assert_param(pDt->Minute < 60);
	assert_param(pDt->Second < 60);
	assert_param(IS_RTC_TF_SET(pDt->TimeFormat));
	assert_param(IS_RTC_AM_PM(pDt->PM));
	
	RTC_UnlockCMD(ENABLE);
	RTC->CR &= 0xFFFFFFFD;
	RTC->CR |= (pDt->TimeFormat);
	
	RTC->CWEEK = pDt->Week;			//(1~7) 1:monday,...,7 sunday
	
	RTC->CYEAR = DEC_ToBCD(pDt->Year);
	RTC->CMONTH = DEC_ToBCD(pDt->Month);
	RTC->CDAY = DEC_ToBCD(pDt->Day);

	RTC->CHOUR = DEC_ToBCD(pDt->Hour)|((pDt->PM)<<8);
	RTC->CMIN = DEC_ToBCD(pDt->Minute);
	RTC->CSEC = DEC_ToBCD(pDt->Second);
}


/**
  * @brief 
  * @param 
  * @retval
  */
void RTC_GetAlarmTime(RTC_Alarm * pDt)
{
	pDt->TimeFormat = (u8)((RTC->CR >> 1) & 0x01);	//0: 24-Hour,1: 12-Hour
	pDt->Hour = (u8)RTC->AHOUR;		
	pDt->Minute = (u8)RTC->AMIN;
	pDt->Second = (u8)RTC->ASEC;
}


/**
  * @brief Set Current Date and Time  
  * @param  pDt :struct of date time -- dec format
  * @retval BCD code
  */
void RTC_SetAlarmTime(RTC_Alarm * pDt)
{
	uint8_t temp;
	assert_param(pDt->Hour < 24);
	assert_param(pDt->Minute < 60);
	assert_param(pDt->Second < 60);
	
	RTC_UnlockCMD(ENABLE);

	temp = DEC_ToBCD(pDt->Second);
	RTC->ASEC = temp;
	temp = DEC_ToBCD(pDt->Minute);
	RTC->AMIN = temp;
	temp = DEC_ToBCD(pDt->Hour);
	RTC->AHOUR = temp;
	
	RTC_UnlockCMD(DISABLE);
}


void RTC_SetTrim(u16 Value)
{
	Value&=0x1ff;
	RTC->TRIM=Value;
}


void RTC_CMD(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState == ENABLE)
	{
		/* Set the update bit */
		RTC->CR |= RTC_CR_EN;
	}
	else
	{
		/* Reset the update bit */
		RTC->CR &= (~RTC_CR_EN);
	}
}



/**
  * @}
  */
