  /******************************************************************************
  * @file    PT32x033x_uart.c
  * @author  ӦÿŶ
  * @version V1.4.0
  * @date    2023/10/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the Universal synchronous asynchronous receiver
  *          transmitter (USART):
  *           + Initialization and Configuration
  *           + STOP Mode
  *           + BaudRate
  *           + Data transfers
  *           + Multi-Processor Communication
  *           + Half-duplex mode
  *           + Smartcard mode
  *           + Interrupts and flags management
  *            
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32x033x_uart.h"


/** @defgroup UART
  * @brief UART driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Initializes the UARTx peripheral according to the specified
  *         parameters in the USART_InitStruct .
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_InitStruct: pointer to a UART_InitTypeDef structure that contains
  *         the configuration information for the specified UART peripheral.
  * @retval None
  */
void UART_Init(UART_TypeDef* UARTx, UART_InitTypeDef* UART_InitStruct)
{
	u32  tmpreg = 0, pclk = 0,quotient=0,remainder=0;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_BR_VAL(UART_InitStruct->UART_BaudRate));
	assert_param(IS_UART_WP_SEL(UART_InitStruct->UART_WordLength));
	assert_param(IS_UART_STOP_SEL(UART_InitStruct->UART_StopBits));
	assert_param(IS_UART_PS_SEL(UART_InitStruct->UART_Parity));
	assert_param(IS_UART_RE_SEL(UART_InitStruct->UART_Mode));
	assert_param(IS_UART_LoopBack_SEL(UART_InitStruct->UART_LoopBack));
	UARTx->RXFR = 0x00;
	UARTx->TXFR = 0x00;
	
	tmpreg &= ~(UART_CR_WP | UART_CR_STOP | UART_Parity_Odd | UART_LoopBack_Enable | UART_Mode_RxEnable | UART_CR_RE);
	tmpreg |= 	(UART_InitStruct ->UART_WordLength | \
				UART_InitStruct ->UART_StopBits | \
				UART_InitStruct ->UART_Parity | \
				UART_InitStruct->UART_LoopBack | \
				UART_InitStruct->UART_Mode);
	UARTx->CR = tmpreg;
	
	pclk = RCC_GetPCLKFreq();
	quotient=(pclk/(16*UART_InitStruct->UART_BaudRate));
	remainder=(pclk%(16*UART_InitStruct->UART_BaudRate));
	if(quotient==0)
	{
		UARTx->BRR=1;
	}
	else
	{
		if(remainder>(8*UART_InitStruct->UART_BaudRate))
		{
			UARTx->BRR = (u16)quotient+1;
		}
		else
		{
			UARTx->BRR = (u16)quotient;	
		}
	}
}


/**
  * @brief  Fills each UART_InitStruct member with its default value.
  * @param  UART_InitStruct: pointer to a UART_InitTypeDef structure
  *         which will be initialized.
  * @retval None
  */
void UART_StructInit(UART_InitTypeDef* UART_InitStruct)
{
	/* USART_InitStruct members default value */
	UART_InitStruct->UART_BaudRate = 19200;
	UART_InitStruct->UART_WordLength = UART_WordLength_8DP;
	UART_InitStruct->UART_StopBits = UART_StopBits_1;
	UART_InitStruct->UART_Parity = UART_Parity_Odd;
	UART_InitStruct->UART_Mode = UART_Mode_RxEnable;
	UART_InitStruct->UART_LoopBack = UART_LoopBack_Disable;
}

/**
  * @brief  Enables or disables the specified UART peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  NewState: new state of the USARTx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_Cmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the selected USART by setting the UE bit in the CR1 register */
		UARTx->CR |= UART_CR_EN;
	}
	else
	{
		/* Disable the selected USART by clearing the UE bit in the CR1 register */
		UARTx->CR &= (uint32_t)~((uint32_t)UART_CR_EN);
	}
}

/**
  * @brief  Enables or disables the UART's  receiver.
  * @param  NewState: new state of the UART transfer direction.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_RxConfig(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the UART's transfer interface as Receiver by setting the RXEN bits
		   in the UART CR register */
		UARTx->CR |= UART_CR_RE;
	}
	else
	{
		/* Disable the UART's transfer interface as Receiver by setting the RXEN bits
		   in the UART CR register */
		UARTx->CR &= (uint32_t)~UART_CR_RE;
	}
}

/**
  * @brief  Sets the receiver Time Out value.
  * @param  UARTx: where x can be 0 or 1 to select the UART peripheral.
  * @param  UART_ReceiverTimeOut: specifies the Receiver Time Out value.
  * @retval None
  */
void UART_SetReceiverTimeOut(UART_TypeDef* UARTx, uint32_t UART_ReceiverTimeOut)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_TIMEOUT(UART_ReceiverTimeOut));
	/* Clear the receiver Time Out value by clearing the RTO[23:0] bits in the RTOR
	   register  */
	UARTx->TO &= (uint32_t)~((uint32_t)UART_TO_TO);
	/* Set the receiver Time Out value by setting the RTO[23:0] bits in the RTOR
	   register  */
	UARTx->TO |= UART_ReceiverTimeOut;
}


/**
  * @brief  Transmits single data through the UARTx peripheral.
  * @param  UARTx: where x can be from 0 or 1 to select the UART peripheral.
  * @param  Data: the data to transmit.
  * @retval None
  */
void UART_SendData(UART_TypeDef* UARTx, u16 Data)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_DATA(Data));
	/* While the TxFIFO contain 8 characters. */
	while((UARTx->SR & UART_SR_TXF));
	/* Transmit Data */
	UARTx->DR = (Data & (u16)0x01FF);
}

/**
  * @brief  Transmits single string through the UARTx peripheral.
  * @param  UARTx: where x can be from 0 or 1 to select the UART peripheral.
  * @param  String: Pointer to the string to transmit.
  * @retval None
  */

void UART_StringSend(UART_TypeDef* UARTx, uint8_t *String)
{
		/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	while(*String != '\0')
	{
		while((UARTx->SR & UART_SR_TXF));
		UART_SendData(UARTx, *String);
		String++;
	}
	UART_SendData(UARTx, 0x00);
}


/**
  * @brief  Returns the most recent received data by the UARTx peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @retval The received data.
  */
u16 UART_ReceiveData(UART_TypeDef* UARTx)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	/* Receive Data */
	return (u16)(UARTx->DR & (u16)0x01FF);
}

/**
  * @brief  Returns the most recent received data by the UARTx peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @Data   Data: Pointer to the receivde data.
  * @param  UART_ReceiverTimeOut: The time out period.
  * @retval The received data.
  */
u16 UART_ReceiveData_Timeout(UART_TypeDef* UARTx, u16 *Data, uint32_t UART_ReceiverTimeOut)
{
	u16 wStatus;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	/* Reload the Timeout counter */
	UART_SetReceiverTimeOut(UARTx, UART_ReceiverTimeOut);
	/* while the RxFIFO is empty and no Timeoutidle */
	while (!((wStatus = UARTx->SR) & (UART_SR_IDLE | UART_SR_RXHF | UART_SR_RXNE))); // while the RxFIFO is empty and no Timeoutidle
	/* Receive Data */
	*Data = (u16)(UARTx->DR & (u16)0x01FF);
	return (wStatus);
}

/**
  * @brief
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @Data   Data: Pointer to the reveive data.
  * @retval The received data.
  */
u16 UART_StringReceive(UART_TypeDef* UARTx, uint8_t *Data)
{
	uint8_t *pSTRING;
	u16 wStatus;
	pSTRING = Data;
		/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	do
	{
		/* while the RxFIFO is empty and no Timeoutidle */
		while (!((wStatus = UARTx->SR) & (UART_SR_RXHF | UART_SR_RXNE))); //
		/* Read the Receive Buffer */
		*(pSTRING++) = (uint8_t)UARTx->DR;
	}
	while((*(pSTRING - 1) != 0x0D) & (*(pSTRING - 1) != '\0'));
	*(pSTRING - 1) = '\0';
	return wStatus;
}

/**
  * @brief  This function the loop back mode of the selected UART.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  NewState: new active level status for the UART pin(s).
  *          This parameter can be:
  *            @arg ENABLE: Loop back mode .
  *            @arg DISABLE: Normal communication mode .
  * @retval The received data.
  */
void UART_LoopBackConfig(UART_TypeDef* UARTx, FunctionalState NewStatus)
{
			/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	if (NewStatus != DISABLE)
	{
		UARTx->CR |= UART_CR_LPB;
	}
	else
	{
		UARTx->CR &= ~UART_CR_LPB;
	}
}

/**
  * @brief  This function reset the Rx and the Tx FIFOs
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  FIFO: Transmit FIFO or receive FIFO for the UART .
  *          This parameter can be:
  *            @arg Rx_FIFO: Receive FIFO .
  *            @arg Tx_FIFO: Transmit FIFO .
  * @retval None.
  */
void UART_FifoReset(UART_TypeDef * UARTx,  UART_FifoTypeDef FIFO)
{
			/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	switch (FIFO)
	{
		case Rx_FIFO :
			UARTx->RXFR = UART_RXFR_RXFR;
			break;
		case Tx_FIFO :
			UARTx->TXFR = UART_TXFR_TXFR;
			break;
	}
}


/**
  * @brief  Enables or disables the UART's IrDA interface.
  * @param  UARTx: where x can be 0 to select the UART peripheral.
  * @param  NewState: new state of the IrDA mode.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_IrDACmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_USART_0_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the IrDA mode by setting the EN bit in the IRR register */
		UARTx->IRC |= UART_IRC_IRE;
	}
	else
	{
		/* Disable the IrDA mode by clearing the EN bit in the IRR register */
		UARTx->IRC &= (uint32_t)~((uint32_t)UART_IRC_IRE);
	}
}

/**
  * @brief  Configures the IrDA's Pin polarity
  * @param  UARTx: where x can be 0 to select the UART peripheral.
  * @param  NewState: new defined levels for the USART data.
  *          This parameter can be:
  *            @arg ENABLE: pin(s) signal values are inverted (Vdd =0, Gnd =1).
  *            @arg pin(s) signal works using the standard logic levels (Vdd =1, Gnd =0).
  * @note   This function has to be called before calling UART_Cmd() function.
  * @retval None
  */
void UART_IrDAInvPinCmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_USART_0_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the binary data inversion feature by setting the DATAINV bit in
		   the CR2 register */
		UARTx->IRC |= UART_IRC_IRPN;
	}
	else
	{
		/* Disable the binary data inversion feature by clearing the DATAINV bit in
		   the CR2 register */
		UARTx->IRC &= (uint32_t)~((uint32_t)UART_IRC_IRPN);
	}
}

/**
  * @brief  Sets the system clock prescaler.
  * @param  USARTx: where x can be 1, 2 or 3 to select the USART peripheral.
  * @param  USART_Prescaler: specifies the prescaler clock.
  * @note   This function has to be called before calling USART_Cmd() function.
  * @retval None
  */
void UART_SetDutyCycle(UART_TypeDef* UARTx, uint8_t UART_DutyCycle)
{
	/* Check the parameters */
	assert_param(IS_USART_0_PERIPH(UARTx));
	assert_param(IS_UART_DUTYCYCLE(UART_DutyCycle));
	/* Clear the IrDA's modulation PWM duty cycle */
	UARTx->IRDC &= ~UART_IRDC_DUTY;
	/* Set the IrDA's modulation PWM duty cycle*/
	UARTx->IRDC |= (uint32_t)((uint32_t)UART_DutyCycle);
}


/**
  * @brief  Enables or disables the UART's Half Duplex communication.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  NewState: new state of the UART Communication.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_HalfDuplexCmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the Half-Duplex mode by setting the OWE bit in the CR register */
		UARTx->CR |= UART_CR_OWE;
	}
	else
	{
		/* Disable the Half-Duplex mode by clearing the OWE bit in the CR register */
		UARTx->CR  &= (uint32_t)~((uint32_t)UART_CR_OWE);
	}
}

/**
  * @brief  Enables or disables the UART's transmitter or receiver.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_Direction: specifies the UART direction.
  *          This parameter can be any combination of the following values:
  *            @arg UART_Mode_Tx: USART Transmitter
  *            @arg UART_Mode_Rx: USART Receiver
  * @param  NewState: new state of the USART transfer direction.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */

void UART_HalfDuplexDirConfig(UART_TypeDef* UARTx, u32 UART_OWD_SEL)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_OWD_SEL(UART_OWD_SEL));
	if (UART_OWD_SEL != UART_OneLine_Rx)
	{
		/* Set NSS pin internally by software */
		UARTx->CR |= UART_OneLine_Tx;
	}
	else
	{
		/* Reset NSS pin internally by software */
		UARTx->CR &= UART_OneLine_Rx;
	}
}

/**
  * @brief  Enables or disables the specified USART interrupts.
  * @param  UARTx: where x can be from 1 to 8 to select the USART peripheral.
  * @param  UART_IT: specifies the UART interrupt sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *            @arg UART_IT_RXNEIE: specifies the interrupt source for Receiver FIFO buffer non-empty interrupt.
  *            @arg UART_IT_RXHFIE: specifies the interrupt source for Receiver FIFO buffer half-full interrupt.
  *            @arg UART_IT_RXFIE: specifies the interrupt source for Receiver FIFO buffer full interrupt.
  *            @arg UART_IT_TONEIE: specifies the interrupt source for Receive Receiver FIFO buffer cleared timeout interrupt.
  *            @arg UART_IT_TOIDLEIE: specifies the interrupt source for Receiver FIFO buffer idel interrupt.
  *            @arg UART_IT_PERRIE: specifies the interrupt source for Parity error interrupt.
  *            @arg UART_IT_FERRIE: specifies the interrupt source for Frame error interrupt.
  *            @arg UART_IT_OVERRIE: specifies the interrupt source for Receiver FIFO buffer overflow interrupt.
  *            @arg UART_IT_TXEIE: specifies the interrupt source for Transmitter FIFO buffer empty interrupt.
  *            @arg UART_IT_TXHEIE: specifies the interrupt source for Transmitter FIFO buffer half-empty interrupt.
  *            @arg UART_IT_TXFIE: specifies the interrupt source for Transmitter FIFO buffer full interrupt.
  *            @arg UART_IT_TXENDIE: specifies the interrupt source for Transfer completed interrupt
  * @param  NewState: new state of the specified USARTx interrupts.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_ITConfig(UART_TypeDef* UARTx, uint32_t UART_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	//assert_param(IS_UART_CONFIG_IT(UART_IT));
	assert_param(IS_UART_IE_SEL(UART_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		UARTx->IE |= UART_IT;
	}
	else
	{
		UARTx->IE &= ~UART_IT;
	}
}


/**
  * @brief  Checks whether the specified UART flag is set or not.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg UART_FLAG_RXNE:  Receiver FIFO buffer non-empty flag.
  *            @arg UART_FLAG_RXHF:  Receiver FIFO buffer half-full flag.
  *            @arg UART_FLAG_RXF:   Receiver FIFO buffer full.
  *            @arg UART_FLAG_TONE:  Receiver FIFO buffer cleared timeout.
  *            @arg UART_FLAG_TOIDLE:  Receiver FIFO buffer idel.
  *            @arg UART_FLAG_PERR:  Parity error.
  *            @arg UART_FLAG_FERR:  Frame error.
  *            @arg UART_FLAG_OVERR: Receiver FIFO buffer overflow.
  *            @arg UART_FLAG_TXE:   Transmitter FIFO buffer empty.
  *            @arg UART_FLAG_TXHE:  Transmitter FIFO buffer half-empty.
  *            @arg UART_FLAG_TXF:   Transmitter FIFO buffer full.
  *            @arg UART_FLAG_TXEND: Transfer completed.
  * @retval The new state of UART_FLAG (SET or RESET).
  */
FlagStatus UART_GetFlagStatus(UART_TypeDef* UARTx, uint32_t UART_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_FLAG(UART_FLAG));
	//assert_param(IS_UART_SR_SEL(UART_FLAG));
	if ((UARTx->SR & UART_FLAG) != (u16)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return bitstatus;
}

/**
  * @brief  Clears the UARTx's pending flags.
  * @param  UARTx: where x can be from 0 to 1 to select the USART peripheral.
  * @param  USART_FLAG: specifies the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg UART_CLEAR_FLAG_PERR:  Parity error.
  *            @arg UART_CLEAR_FLAG_FERR:  Frame error.
  *            @arg UART_CLEAR_FLAG_OVERR: Receiver FIFO buffer overflow.
  * @retval None
  */
void UART_ClearFlag(UART_TypeDef* UARTx, uint32_t UART_CLEAR_FLAG)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_IT_SEL(UART_CLEAR_FLAG));
	UARTx->SR = UART_CLEAR_FLAG;
}

/**
  * @brief
  * @param
  * @param
  * @retval
  */
ITStatus UART_GetITStatus(UART_TypeDef* UARTx, uint32_t UART_IT)
{
	ITStatus bitstatus = RESET;
	uint32_t enablestatus = 0;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_IT_GET(UART_IT));
	/**/
	enablestatus = (uint32_t)(UARTx->IE & UART_IT);
	/*  */
	if (((uint32_t)(UARTx->SR & UART_IT) != (uint32_t)RESET) && (enablestatus != (uint32_t)RESET))
	{
		/* */
		bitstatus = SET;
	}
	else
	{
		/* */
		bitstatus = RESET;
	}
	/**/
	return  bitstatus;
}

/**
  * @}
  */

